@@ -131,6 +131,8 @@ class WritableCatalogManager : public SimpleCatalogManager {
                            const bool merge = true);
   void SwapNestedCatalog(const string &mountpoint, const shash::Any &new_hash,
                          const uint64_t new_size);
+  void GraftNestedCatalog(const string &mountpoint, const shash::Any &new_hash,
+                          const uint64_t new_size);
   bool IsTransitionPoint(const std::string &mountpoint);
   WritableCatalog *GetHostingCatalog(const std::string &path);
 
