@@ -2,7 +2,7 @@
  * This file is part of the CernVM file system.
  */
 
-#include "cvmfs_config.h"
+
 #include "catalog_sql.h"
 
 #include <cstdlib>
@@ -10,7 +10,7 @@
 
 #include "catalog.h"
 #include "globals.h"
-#include "logging.h"
+#include "util/logging.h"
 #include "util/posix.h"
 #include "xattr.h"
 
@@ -25,6 +25,9 @@ namespace catalog {
  *
  * Repository rollbacks to an outdated catalog schema is not supported. Have a
  * look into CVM-252 if that becomes necessary at some point.
+ *
+ * If the statistics counters get modified, the swissknife migration 'stats'
+ * tools needs to be modified, too.
  * NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE
  */
 
@@ -49,7 +52,7 @@ namespace catalog {
 //
 // 1.x (earlier - code base still in SVN)
 //     * pre-historic times
-// 0.9 (some time 2011, artifical version)
+// 0.9 (some time 2011, artificial version)
 //     * 1.0 catalogs that lack the SHA-1 value for nested catalogs
 const float CatalogDatabase::kLatestSchema = 2.5;
 const float CatalogDatabase::kLatestSupportedSchema = 2.5;  // + 1.X (r/o)
@@ -72,7 +75,13 @@ const float CatalogDatabase::kLatestSupportedSchema = 2.5;  // + 1.X (r/o)
 //   4 --> 5: (Dec 07 2017):
 //            * add kFlagFileSpecial (rebranded unused kFlagFileStat)
 //            * add self_special and subtree_special statistics counters
-const unsigned CatalogDatabase::kLatestSchemaRevision = 5;
+//   5 --> 6: (Jul 01 2021):
+//            * Add kFlagDirectIo
+//   6 --> 7: (Feb 23 2024):
+//            * Store nanosecond timestamps (mtimens column) in catalog table.
+//              The mtimens column has only the nanosecond part of the timestamp
+//              and may be NULL
+const unsigned CatalogDatabase::kLatestSchemaRevision = 7;
 
 bool CatalogDatabase::CheckSchemaCompatibility() {
   return !( (schema_version() >= 2.0-kSchemaEpsilon)                   &&
@@ -193,6 +202,33 @@ bool CatalogDatabase::LiveSchemaUpgradeIfNecessary() {
     }
   }
 
+
+  if (IsEqualSchema(schema_version(), 2.5) && (schema_revision() == 5)) {
+    LogCvmfs(kLogCatalog, kLogDebug, "upgrading schema revision (5 --> 6)");
+
+    set_schema_revision(6);
+    if (!StoreSchemaRevision()) {
+      LogCvmfs(kLogCatalog, kLogDebug, "failed to upgrade schema revision");
+      return false;
+    }
+  }
+
+  if (IsEqualSchema(schema_version(), 2.5) && (schema_revision() == 6)) {
+    LogCvmfs(kLogCatalog, kLogDebug, "upgrading schema revision (6 --> 7)");
+
+    SqlCatalog sql_upgrade1(*this, "ALTER TABLE catalog ADD mtimens INTEGER;");
+    if (!sql_upgrade1.Execute()) {
+      LogCvmfs(kLogCatalog, kLogDebug, "failed to upgrade catalogs (6 --> 7)");
+      return false;
+    }
+
+    set_schema_revision(7);
+    if (!StoreSchemaRevision()) {
+      LogCvmfs(kLogCatalog, kLogDebug, "failed to upgrade schema revision");
+      return false;
+    }
+  }
+
   return true;
 }
 
@@ -206,8 +242,8 @@ bool CatalogDatabase::CreateEmptyDatabase() {
     "CREATE TABLE catalog "
     "(md5path_1 INTEGER, md5path_2 INTEGER, parent_1 INTEGER, parent_2 INTEGER,"
     " hardlinks INTEGER, hash BLOB, size INTEGER, mode INTEGER, mtime INTEGER,"
-    " flags INTEGER, name TEXT, symlink TEXT, uid INTEGER, gid INTEGER, "
-    " xattr BLOB, "
+    " mtimens INTEGER, flags INTEGER, name TEXT, symlink TEXT, uid INTEGER,"
+    " gid INTEGER, xattr BLOB, "
     " CONSTRAINT pk_catalog PRIMARY KEY (md5path_1, md5path_2));").Execute()  &&
   SqlCatalog(*this,
     "CREATE INDEX idx_catalog_parent "
@@ -421,6 +457,8 @@ unsigned SqlDirent::CreateDatabaseFlags(const DirectoryEntry &entry) const {
       database_flags |= kFlagFileChunk;
     if (entry.IsExternalFile())
       database_flags |= kFlagFileExternal;
+    if (entry.IsDirectIo())
+      database_flags |= kFlagDirectIo;
   }
 
   if (!entry.checksum_ptr()->IsNull() || entry.IsChunkedFile())
@@ -549,6 +587,7 @@ bool SqlDirentWrite::BindDirentFields(const int hash_idx,
                                       const int size_idx,
                                       const int mode_idx,
                                       const int mtime_idx,
+                                      const int mtimens_idx,
                                       const int flags_idx,
                                       const int name_idx,
                                       const int symlink_idx,
@@ -560,7 +599,7 @@ bool SqlDirentWrite::BindDirentFields(const int hash_idx,
     MakeHardlinks(entry.hardlink_group_,
                   entry.linkcount_);
 
-  return (
+  bool result =
     BindHashBlob(hash_idx, entry.checksum_) &&
     BindInt64(hardlinks_idx, hardlinks) &&
     BindInt64(size_idx, entry.size_) &&
@@ -569,9 +608,17 @@ bool SqlDirentWrite::BindDirentFields(const int hash_idx,
     BindInt64(gid_idx, entry.gid_) &&
     BindInt64(mtime_idx, entry.mtime_) &&
     BindInt(flags_idx, CreateDatabaseFlags(entry)) &&
-    BindText(name_idx, entry.name_.GetChars(), entry.name_.GetLength()) &&
-    BindText(symlink_idx, entry.symlink_.GetChars(), entry.symlink_.GetLength())
-  );  // NOLINT
+    BindText(name_idx, entry.name_.GetChars(),
+                       static_cast<int>(entry.name_.GetLength())) &&
+    BindText(symlink_idx, entry.symlink_.GetChars(),
+                          static_cast<int>(entry.symlink_.GetLength()));
+  if (entry.HasMtimeNs()) {
+    result &= BindInt(mtimens_idx, entry.mtime_ns_);
+  } else {
+    result &= BindNull(mtimens_idx);
+  }
+
+  return result;
 }
 
 
@@ -631,14 +678,21 @@ shash::Any SqlListContentHashes::GetHash() const {
            "catalog.name,       catalog.symlink,    catalog.md5path_1, "  \
            "catalog.md5path_2,  catalog.parent_1,   catalog.parent_2, "   \
            "catalog.rowid,      catalog.uid,        catalog.gid, "        \
-           "0"
-#define DB_FIELDS_GE_V2_1_GE_R2                                           \
+           "0, NULL"
+#define DB_FIELDS_GE_V2_1_LT_R7                                           \
            "catalog.hash,       catalog.hardlinks,  catalog.size, "       \
            "catalog.mode,       catalog.mtime,      catalog.flags, "      \
            "catalog.name,       catalog.symlink,    catalog.md5path_1, "  \
            "catalog.md5path_2,  catalog.parent_1,   catalog.parent_2, "   \
            "catalog.rowid,      catalog.uid,        catalog.gid, "        \
-           "catalog.xattr IS NOT NULL"
+           "catalog.xattr IS NOT NULL, NULL"
+#define DB_FIELDS_GE_V2_1_GE_R7                                           \
+           "catalog.hash,       catalog.hardlinks,  catalog.size, "       \
+           "catalog.mode,       catalog.mtime,      catalog.flags, "      \
+           "catalog.name,       catalog.symlink,    catalog.md5path_1, "  \
+           "catalog.md5path_2,  catalog.parent_1,   catalog.parent_2, "   \
+           "catalog.rowid,      catalog.uid,        catalog.gid, "        \
+           "catalog.xattr IS NOT NULL, catalog.mtimens"
 
 #define MAKE_STATEMENT(STMT_TMPL, REV)                        \
   static const std::string REV =                              \
@@ -647,7 +701,8 @@ shash::Any SqlListContentHashes::GetHash() const {
 #define MAKE_STATEMENTS(STMT_TMPL)          \
   MAKE_STATEMENT(STMT_TMPL, LT_V2_1);       \
   MAKE_STATEMENT(STMT_TMPL, GE_V2_1_LT_R2); \
-  MAKE_STATEMENT(STMT_TMPL, GE_V2_1_GE_R2)
+  MAKE_STATEMENT(STMT_TMPL, GE_V2_1_LT_R7); \
+  MAKE_STATEMENT(STMT_TMPL, GE_V2_1_GE_R7)
 
 #define DEFERRED_INIT(DB, REV) \
   DeferredInit((DB).sqlite_db(), (REV).c_str())
@@ -657,8 +712,10 @@ shash::Any SqlListContentHashes::GetHash() const {
     DEFERRED_INIT((DB), LT_V2_1);                                        \
   } else if ((DB).schema_revision() < 2) {                               \
     DEFERRED_INIT((DB), GE_V2_1_LT_R2);                                  \
+  } else if ((DB).schema_revision() < 7) {                               \
+    DEFERRED_INIT((DB), GE_V2_1_LT_R7);                                  \
   } else {                                                               \
-    DEFERRED_INIT((DB), GE_V2_1_GE_R2);                                  \
+    DEFERRED_INIT((DB), GE_V2_1_GE_R7);                                  \
   }
 
 
@@ -706,8 +763,10 @@ DirectoryEntry SqlLookup::GetDirent(const Catalog *catalog,
     result.is_bind_mountpoint_ = (database_flags & kFlagDirBindMountpoint);
     result.is_chunked_file_    = (database_flags & kFlagFileChunk);
     result.is_hidden_          = (database_flags & kFlagHidden);
+    result.is_direct_io_       = (database_flags & kFlagDirectIo);
     result.is_external_file_   = (database_flags & kFlagFileExternal);
     result.has_xattrs_         = RetrieveInt(15) != 0;
+    result.mtime_ns_           = RetrieveNullableInt(16, -1);
     result.checksum_           =
       RetrieveHashBlob(0, RetrieveHashAlgorithm(database_flags));
     result.compression_algorithm_ =
@@ -730,6 +789,14 @@ DirectoryEntry SqlLookup::GetDirent(const Catalog *catalog,
   if (expand_symlink && !g_raw_symlinks)
     ExpandSymlink(&result.symlink_);
 
+  if (g_world_readable) {
+    if (S_ISDIR(result.mode_)) {
+       result.mode_ |= 0555;
+    } else {
+       result.mode_ |= 0444;
+    }
+  }
+
   return result;
 }
 
@@ -804,15 +871,17 @@ SqlDirentTouch::SqlDirentTouch(const CatalogDatabase &database) {
     "UPDATE catalog "
     "SET hash = :hash, size = :size, mode = :mode, mtime = :mtime, "
 //            1             2             3               4
-    "name = :name, symlink = :symlink, uid = :uid, gid = :gid, xattr = :xattr "
+    "name = :name, symlink = :symlink, uid = :uid, gid = :gid, xattr = :xattr, "
 //        5                6               7           8             9
+    "mtimens = :mtimens "
+//             10
     "WHERE (md5path_1 = :md5_1) AND (md5path_2 = :md5_2);");
-//                    10                       11
+//                    11                       12
 }
 
 
 bool SqlDirentTouch::BindDirentBase(const DirectoryEntryBase &entry) {
-  return (
+  bool result =
     BindHashBlob(1, entry.checksum_) &&
     BindInt64(2, entry.size_) &&
     BindInt(3, entry.mode_) &&
@@ -820,12 +889,18 @@ bool SqlDirentTouch::BindDirentBase(const DirectoryEntryBase &entry) {
     BindText(5, entry.name_.GetChars(),    entry.name_.GetLength()) &&
     BindText(6, entry.symlink_.GetChars(), entry.symlink_.GetLength()) &&
     BindInt64(7, entry.uid_) &&
-    BindInt64(8, entry.gid_));
+    BindInt64(8, entry.gid_);
+  if (entry.HasMtimeNs()) {
+    result &= BindInt(10, entry.mtime_ns_);
+  } else {
+    result &= BindNull(10);
+  }
+  return result;
 }
 
 
 bool SqlDirentTouch::BindPathHash(const shash::Md5 &hash) {
-  return BindMd5(10, 11, hash);
+  return BindMd5(11, 12, hash);
 }
 
 
@@ -1008,10 +1083,10 @@ SqlDirentInsert::SqlDirentInsert(const CatalogDatabase &database) {
     "INSERT INTO catalog "
     "(md5path_1, md5path_2, parent_1, parent_2, hash, hardlinks, size, mode,"
     //    1           2         3         4       5       6        7     8
-    "mtime, flags, name, symlink, uid, gid, xattr) "
-    // 9,     10    11     12     13   14   15
+    "mtime, flags, name, symlink, uid, gid, xattr, mtimens) "
+    // 9,     10    11     12     13   14   15     16
     "VALUES (:md5_1, :md5_2, :p_1, :p_2, :hash, :links, :size, :mode, :mtime,"
-    " :flags, :name, :symlink, :uid, :gid, :xattr);");
+    " :flags, :name, :symlink, :uid, :gid, :xattr, :mtimens);");
 }
 
 
@@ -1026,7 +1101,7 @@ bool SqlDirentInsert::BindParentPathHash(const shash::Md5 &hash) {
 
 
 bool SqlDirentInsert::BindDirent(const DirectoryEntry &entry) {
-  return BindDirentFields(5, 6, 7, 8, 9, 10, 11, 12, 13, 14, entry);
+  return BindDirentFields(5, 6, 7, 8, 9, 16, 10, 11, 12, 13, 14, entry);
 }
 
 
@@ -1055,20 +1130,20 @@ SqlDirentUpdate::SqlDirentUpdate(const CatalogDatabase &database) {
 //            1             2             3               4
     "flags = :flags, name = :name, symlink = :symlink, hardlinks = :hardlinks, "
 //          5             6                  7                8
-    "uid = :uid, gid = :gid "
-//          9           10
+    "uid = :uid, gid = :gid, mtimens = :mtimens "
+//          9           10             11
     "WHERE (md5path_1 = :md5_1) AND (md5path_2 = :md5_2);");
-//                     11                       12
+//                     12                       13
 }
 
 
 bool SqlDirentUpdate::BindPathHash(const shash::Md5 &hash) {
-  return BindMd5(11, 12, hash);
+  return BindMd5(12, 13, hash);
 }
 
 
 bool SqlDirentUpdate::BindDirent(const DirectoryEntry &entry) {
-  return BindDirentFields(1, 8, 2, 3, 4, 5, 6, 7, 9, 10, entry);
+  return BindDirentFields(1, 8, 2, 3, 4, 11, 5, 6, 7, 9, 10, entry);
 }
 
 
