@@ -24,10 +24,10 @@
 
 #include <string>
 
-#include "compression.h"
+#include "compression/compression.h"
+#include "crypto/hash.h"
 #include "directory_entry.h"
 #include "file_chunk.h"
-#include "hash.h"
 #include "shortstring.h"
 #include "sql.h"
 
@@ -73,7 +73,7 @@ class CatalogDatabase : public sqlite::Database<CatalogDatabase> {
 
 /**
  * Base class for all SQL statement classes.  It wraps a single SQL statement
- * and all neccessary calls of the sqlite3 API to deal with this statement.
+ * and all necessary calls of the sqlite3 API to deal with this statement.
  */
 class SqlCatalog : public sqlite::Sql {
  public:
@@ -140,7 +140,8 @@ class SqlCatalog : public sqlite::Sql {
   {
     uint64_t high, low;
     hash.ToIntPair(&high, &low);
-    const bool retval = BindInt64(idx_high, high) && BindInt64(idx_low, low);
+    const bool retval = BindInt64(idx_high, static_cast<int64_t>(high)) &&
+                        BindInt64(idx_low, static_cast<int64_t>(low));
     return retval;
   }
 
@@ -168,7 +169,7 @@ class SqlCatalog : public sqlite::Sql {
 
 
 /**
- * Common ancestor of SQL statemnts that deal with directory entries.
+ * Common ancestor of SQL statements that deal with directory entries.
  */
 class SqlDirent : public SqlCatalog {
  public:
@@ -209,6 +210,10 @@ class SqlDirent : public SqlCatalog {
    * directory.
    */
   static const int kFlagHidden              = 0x8000;  // 2^15
+  /**
+   * For regular files, indicates that the file should be opened with direct I/O
+   */
+  static const int kFlagDirectIo            = 0x10000;  // 2^16
 
 
  protected:
@@ -260,6 +265,7 @@ class SqlDirentWrite : public SqlDirent {
                         const int size_idx,
                         const int mode_idx,
                         const int mtime_idx,
+                        const int mtimens_idx,
                         const int flags_idx,
                         const int name_idx,
                         const int symlink_idx,
