@@ -48,12 +48,12 @@
 #include <map>
 #include <string>
 
-#include "atomic.h"
-#include "platform.h"
 #include "smallhash.h"
-#include "smalloc.h"
 #include "statistics.h"
+#include "util/atomic.h"
+#include "util/platform.h"
 #include "util/single_copy.h"
+#include "util/smalloc.h"
 
 namespace lru {
 
