@@ -85,7 +85,7 @@ class Watchdog : SingleCopy {
 
   static void *MainWatchdogListener(void *data);
 
-  static void ReportSignalAndTerminate(int sig, siginfo_t *siginfo,
+  static void ReportSignalAndContinue(int sig, siginfo_t *siginfo,
                                        void *context);
   static void SendTrace(int sig, siginfo_t *siginfo, void *context);
 
