@@ -8,6 +8,7 @@
 #define CVMFS_MOUNTPOINT_H_
 
 #include <pthread.h>
+#include <sys/statvfs.h>
 #include <unistd.h>
 
 #include <ctime>
@@ -16,10 +17,11 @@
 #include <vector>
 
 #include "cache.h"
+#include "crypto/hash.h"
 #include "file_watcher.h"
 #include "gtest/gtest_prod.h"
-#include "hash.h"
 #include "loader.h"
+#include "magic_xattr.h"
 #include "util/algorithm.h"
 #include "util/pointer.h"
 
@@ -42,7 +44,8 @@ class DownloadManager;
 }
 namespace glue {
 class InodeTracker;
-class NentryTracker;
+class DentryTracker;
+class PageCacheTracker;
 }
 namespace lru {
 class InodeCache;
@@ -54,6 +57,7 @@ class OptionsManager;
 namespace perf {
 class Counter;
 class Statistics;
+class TelemetryAggregator;
 }
 namespace signature {
 class SignatureManager;
@@ -148,6 +152,26 @@ class FileSystem : SingleCopy, public BootFactory {
     bool foreground;
   };
 
+  /**
+   * Keeps information about I/O errors, e.g. writing local files, permanent
+   * network errors, etc. It counts the number of errors and the timestamp
+   * of the latest errors for consumption by monitoring tools such as Nagios
+   */
+  class IoErrorInfo {
+   public:
+    IoErrorInfo();
+
+    void Reset();
+    void AddIoError();
+    void SetCounter(perf::Counter *c);
+    int64_t count();
+    time_t timestamp_last();
+
+   private:
+    perf::Counter *counter_;
+    time_t timestamp_last_;
+  };
+
   /**
    * No NFS maps.
    */
@@ -164,18 +188,27 @@ class FileSystem : SingleCopy, public BootFactory {
   static FileSystem *Create(const FileSystemInfo &fs_info);
   ~FileSystem();
 
+  // Used to setup logging before the file system object is created
+  static void SetupLoggingStandalone(
+    const OptionsManager &options_mgr, const std::string &prefix);
+
   bool IsNfsSource() { return nfs_mode_ & kNfsMaps; }
   bool IsHaNfsSource() { return nfs_mode_ & kNfsMapsHa; }
   void ResetErrorCounters();
   void TearDown2ReadOnly();
   void RemapCatalogFd(int from, int to);
 
+  // Used in cvmfs' RestoreState to prevent change of cache manager type
+  // during reload
+  void ReplaceCacheManager(CacheManager *new_cache_mgr);
+
   CacheManager *cache_mgr() { return cache_mgr_; }
   std::string cache_mgr_instance() { return cache_mgr_instance_; }
   std::string exe_path() { return exe_path_; }
   bool found_previous_crash() { return found_previous_crash_; }
   Log2Histogram *hist_fs_lookup() { return hist_fs_lookup_; }
   Log2Histogram *hist_fs_forget() { return hist_fs_forget_; }
+  Log2Histogram *hist_fs_forget_multi() { return hist_fs_forget_multi_; }
   Log2Histogram *hist_fs_getattr() { return hist_fs_getattr_; }
   Log2Histogram *hist_fs_readlink() { return hist_fs_readlink_; }
   Log2Histogram *hist_fs_opendir() { return hist_fs_opendir_; }
@@ -187,17 +220,31 @@ class FileSystem : SingleCopy, public BootFactory {
 
   perf::Counter *n_fs_dir_open() { return n_fs_dir_open_; }
   perf::Counter *n_fs_forget() { return n_fs_forget_; }
+  perf::Counter *n_fs_inode_replace() { return n_fs_inode_replace_; }
   perf::Counter *n_fs_lookup() { return n_fs_lookup_; }
   perf::Counter *n_fs_lookup_negative() { return n_fs_lookup_negative_; }
   perf::Counter *n_fs_open() { return n_fs_open_; }
   perf::Counter *n_fs_read() { return n_fs_read_; }
   perf::Counter *n_fs_readlink() { return n_fs_readlink_; }
   perf::Counter *n_fs_stat() { return n_fs_stat_; }
-  perf::Counter *n_io_error() { return n_io_error_; }
+  perf::Counter *n_fs_stat_stale() { return n_fs_stat_stale_; }
+  perf::Counter *n_fs_statfs() { return n_fs_statfs_; }
+  perf::Counter *n_fs_statfs_cached() { return n_fs_statfs_cached_; }
+  IoErrorInfo *io_error_info() { return &io_error_info_; }
   std::string name() { return name_; }
   NfsMaps *nfs_maps() { return nfs_maps_; }
   perf::Counter *no_open_dirs() { return no_open_dirs_; }
   perf::Counter *no_open_files() { return no_open_files_; }
+  perf::Counter *n_eio_total() { return n_eio_total_; }
+  perf::Counter *n_eio_01() { return n_eio_01_; }
+  perf::Counter *n_eio_02() { return n_eio_02_; }
+  perf::Counter *n_eio_03() { return n_eio_03_; }
+  perf::Counter *n_eio_04() { return n_eio_04_; }
+  perf::Counter *n_eio_05() { return n_eio_05_; }
+  perf::Counter *n_eio_06() { return n_eio_06_; }
+  perf::Counter *n_eio_07() { return n_eio_07_; }
+  perf::Counter *n_eio_08() { return n_eio_08_; }
+  perf::Counter *n_emfile()  { return n_emfile_; }
   OptionsManager *options_mgr() { return options_mgr_; }
   perf::Statistics *statistics() { return statistics_; }
   Type type() { return type_; }
@@ -218,7 +265,7 @@ class FileSystem : SingleCopy, public BootFactory {
     PosixCacheSettings() :
       is_shared(false), is_alien(false), is_managed(false),
       avoid_rename(false), cache_base_defined(false), cache_dir_defined(false),
-      quota_limit(0)
+      quota_limit(0), do_refcount(true)
       { }
     bool is_shared;
     bool is_alien;
@@ -231,6 +278,7 @@ class FileSystem : SingleCopy, public BootFactory {
      * cache when the limit is exceeded.
      */
     int64_t quota_limit;
+    bool do_refcount;
     std::string cache_path;
     /**
      * Different from cache_path only if CVMFS_WORKSPACE or
@@ -245,6 +293,7 @@ class FileSystem : SingleCopy, public BootFactory {
 
   explicit FileSystem(const FileSystemInfo &fs_info);
 
+  static void SetupGlobalEnvironmentParams();
   void SetupLogging();
   void CreateStatistics();
   void SetupSqlite();
@@ -286,16 +335,31 @@ class FileSystem : SingleCopy, public BootFactory {
   perf::Counter *n_fs_lookup_;
   perf::Counter *n_fs_lookup_negative_;
   perf::Counter *n_fs_stat_;
+  perf::Counter *n_fs_stat_stale_;
+  perf::Counter *n_fs_statfs_;
+  perf::Counter *n_fs_statfs_cached_;
   perf::Counter *n_fs_read_;
   perf::Counter *n_fs_readlink_;
   perf::Counter *n_fs_forget_;
-  perf::Counter *n_io_error_;
+  perf::Counter *n_fs_inode_replace_;
   perf::Counter *no_open_files_;
   perf::Counter *no_open_dirs_;
+  perf::Counter *n_eio_total_;
+  perf::Counter *n_eio_01_;
+  perf::Counter *n_eio_02_;
+  perf::Counter *n_eio_03_;
+  perf::Counter *n_eio_04_;
+  perf::Counter *n_eio_05_;
+  perf::Counter *n_eio_06_;
+  perf::Counter *n_eio_07_;
+  perf::Counter *n_eio_08_;
+  perf::Counter *n_emfile_;
+  IoErrorInfo io_error_info_;
   perf::Statistics *statistics_;
 
   Log2Histogram *hist_fs_lookup_;
   Log2Histogram *hist_fs_forget_;
+  Log2Histogram *hist_fs_forget_multi_;
   Log2Histogram *hist_fs_getattr_;
   Log2Histogram *hist_fs_readlink_;
   Log2Histogram *hist_fs_opendir_;
@@ -371,6 +435,39 @@ class FileSystem : SingleCopy, public BootFactory {
   bool has_custom_sqlitevfs_;
 };
 
+/**
+ * The StatfsCache class is a class purely designed as "struct" (= holding
+ * object for all its parameters).
+ * All its logic, including the locking mechanism, is implemented in the
+ * function cvmfs_statfs in cvmfs.cc
+ */
+class StatfsCache : SingleCopy {
+ public:
+  explicit StatfsCache(uint64_t cacheValid) : expiry_deadline_(0),
+                                     cache_timeout_(cacheValid) {
+    memset(&info_, 0, sizeof(info_));
+    lock_ =
+      reinterpret_cast<pthread_mutex_t *>(smalloc(sizeof(pthread_mutex_t)));
+    int retval = pthread_mutex_init(lock_, NULL);
+    assert(retval == 0);
+  }
+  ~StatfsCache() {
+    pthread_mutex_destroy(lock_);
+    free(lock_);
+  }
+  uint64_t *expiry_deadline() { return &expiry_deadline_; }
+  const uint64_t cache_timeout() { return cache_timeout_; }
+  struct statvfs *info() { return &info_; }
+  pthread_mutex_t *lock() { return lock_; }
+
+ private:
+  pthread_mutex_t *lock_;
+  // Timestamp/deadline when the currently cached statvfs info_ becomes invalid
+  uint64_t expiry_deadline_;
+  // Time in seconds how long statvfs info_ should be cached
+  uint64_t cache_timeout_;
+  struct statvfs info_;
+};
 
 /**
  * A MountPoint provides a clip around all the different *Manager objects that
@@ -415,11 +512,14 @@ class MountPoint : SingleCopy, public BootFactory {
   cvmfs::Fetcher *fetcher() { return fetcher_; }
   bool fixed_catalog() { return fixed_catalog_; }
   std::string fqrn() const { return fqrn_; }
+  // TODO(jblomer): use only a singler fetcher object
   cvmfs::Fetcher *external_fetcher() { return external_fetcher_; }
   FileSystem *file_system() { return file_system_; }
+  MagicXattrManager *magic_xattr_mgr() { return magic_xattr_mgr_; }
   bool has_membership_req() { return has_membership_req_; }
-  bool hide_magic_xattrs() { return hide_magic_xattrs_; }
   bool enforce_acls() { return enforce_acls_; }
+  bool cache_symlinks() { return cache_symlinks_; }
+  bool fuse_expire_entry() { return fuse_expire_entry_; }
   catalog::InodeAnnotation *inode_annotation() {
     return inode_annotation_;
   }
@@ -428,16 +528,24 @@ class MountPoint : SingleCopy, public BootFactory {
   double kcache_timeout_sec() { return kcache_timeout_sec_; }
   lru::Md5PathCache *md5path_cache() { return md5path_cache_; }
   std::string membership_req() { return membership_req_; }
-  glue::NentryTracker *nentry_tracker() { return nentry_tracker_; }
+  glue::DentryTracker *dentry_tracker() { return dentry_tracker_; }
+  glue::PageCacheTracker *page_cache_tracker() { return page_cache_tracker_; }
   lru::PathCache *path_cache() { return path_cache_; }
   std::string repository_tag() { return repository_tag_; }
   SimpleChunkTables *simple_chunk_tables() { return simple_chunk_tables_; }
   perf::Statistics *statistics() { return statistics_; }
+  perf::TelemetryAggregator *telemetry_aggr() { return telemetry_aggr_; }
   signature::SignatureManager *signature_mgr() { return signature_mgr_; }
+  uid_t talk_socket_uid() { return talk_socket_uid_; }
+  gid_t talk_socket_gid() { return talk_socket_gid_; }
+  std::string talk_socket_path() { return talk_socket_path_; }
   Tracer *tracer() { return tracer_; }
   cvmfs::Uuid *uuid() { return uuid_; }
+  StatfsCache *statfs_cache() { return statfs_cache_; }
 
   bool ReloadBlacklists();
+  void DisableCacheSymlinks();
+  void EnableFuseExpireEntry();
 
  private:
   /**
@@ -482,6 +590,11 @@ class MountPoint : SingleCopy, public BootFactory {
   static const unsigned kTracerBufferSize = 8192;
   static const unsigned kTracerFlushThreshold = 7000;
   static const char *kDefaultBlacklist;  // "/etc/cvmfs/blacklist"
+  /**
+   * Default values for telemetry aggregator
+  */
+  static const int kDefaultTelemetrySendRateSec = 5 * 60;  // 5min
+  static const int kMinimumTelemetrySendRateSec = 5;  // 5sec
 
   MountPoint(const std::string &fqrn,
              FileSystem *file_system,
@@ -497,7 +610,7 @@ class MountPoint : SingleCopy, public BootFactory {
   bool CreateCatalogManager();
   void CreateTables();
   bool CreateTracer();
-  void SetupBehavior();
+  bool SetupBehavior();
   void SetupDnsTuning(download::DownloadManager *manager);
   void SetupHttpTuning();
   bool SetupExternalDownloadMgr(bool dogeosort);
@@ -520,6 +633,7 @@ class MountPoint : SingleCopy, public BootFactory {
   OptionsManager *options_mgr_;
 
   perf::Statistics *statistics_;
+  perf::TelemetryAggregator *telemetry_aggr_;
   AuthzFetcher *authz_fetcher_;
   AuthzSessionManager *authz_session_mgr_;
   AuthzAttachment *authz_attachment_;
@@ -538,7 +652,10 @@ class MountPoint : SingleCopy, public BootFactory {
   lru::Md5PathCache *md5path_cache_;
   Tracer *tracer_;
   glue::InodeTracker *inode_tracker_;
-  glue::NentryTracker *nentry_tracker_;
+  glue::DentryTracker *dentry_tracker_;
+  glue::PageCacheTracker *page_cache_tracker_;
+  MagicXattrManager *magic_xattr_mgr_;
+  StatfsCache *statfs_cache_;
 
   file_watcher::FileWatcher* resolv_conf_watcher_;
 
@@ -546,14 +663,19 @@ class MountPoint : SingleCopy, public BootFactory {
   pthread_mutex_t lock_max_ttl_;
   double kcache_timeout_sec_;
   bool fixed_catalog_;
-  bool hide_magic_xattrs_;
   bool enforce_acls_;
+  bool cache_symlinks_;
+  bool fuse_expire_entry_;
   std::string repository_tag_;
   std::vector<std::string> blacklist_paths_;
 
   // TODO(jblomer): this should go in the catalog manager
   std::string membership_req_;
   bool has_membership_req_;
+
+  std::string talk_socket_path_;
+  uid_t talk_socket_uid_;
+  gid_t talk_socket_gid_;
 };  // class MointPoint
 
 #endif  // CVMFS_MOUNTPOINT_H_
