@@ -10,12 +10,12 @@
 #include <string>
 
 #include "catalog.h"
-#include "download.h"
+#include "crypto/signature.h"
 #include "history_sqlite.h"
 #include "manifest.h"
 #include "manifest_fetch.h"
+#include "network/download.h"
 #include "reflog.h"
-#include "signature.h"
 #include "util/posix.h"
 
 /**
@@ -377,7 +377,7 @@ class LocalObjectFetcher :
 
   Failures Fetch(const std::string &relative_path,
                  const bool         decompress,
-                 const bool         nocache,
+                 const bool         /* nocache */,
                        std::string *file_path) {
     assert(file_path != NULL);
     file_path->clear();
@@ -582,12 +582,10 @@ class HttpObjectFetcher :
     // fetch and decompress the requested object
     const std::string url = BuildUrl(relative_path);
     const bool probe_hosts = false;
-    download::JobInfo download_job(&url,
-                                        decompress,
-                                        probe_hosts,
-                                        f,
-                                        expected_hash);
-    download_job.force_nocache = nocache;
+    cvmfs::FileSink filesink(f);
+    download::JobInfo download_job(&url, decompress, probe_hosts, expected_hash,
+                                   &filesink);
+    download_job.SetForceNocache(nocache);
     download::Failures retval = download_manager_->Fetch(&download_job);
     const bool success = (retval == download::kFailOk);
     fclose(f);
@@ -610,13 +608,18 @@ class HttpObjectFetcher :
         case download::kFailProxyResolve:
         case download::kFailHostResolve:
         case download::kFailUnsupportedProtocol:
+          LogCvmfs(kLogDownload, kLogDebug | kLogStderr,
+                   "HTTP connection error %d: %s", retval, url.c_str());
           return BaseTN::kFailNetwork;
 
         case download::kFailProxyHttp:
         case download::kFailHostHttp:
-          return (download_job.http_code == 404)
-            ? BaseTN::kFailNotFound
-            : BaseTN::kFailNetwork;
+          if (download_job.http_code() == 404)
+            return BaseTN::kFailNotFound;
+          LogCvmfs(kLogDownload, kLogDebug | kLogStderr,
+                   "HTTP protocol error %d: %s (%d)",
+                   download_job.http_code(), url.c_str(), retval);
+          return BaseTN::kFailNetwork;
 
         case download::kFailBadData:
         case download::kFailTooBig:
@@ -624,7 +627,11 @@ class HttpObjectFetcher :
 
         default:
           if (download::IsProxyTransferError(retval) ||
-              download::IsHostTransferError(retval)) {
+              download::IsHostTransferError(retval))
+          {
+            LogCvmfs(kLogDownload, kLogDebug | kLogStderr,
+                     "HTTP transfer error %d (HTTP code %d): %s",
+                     retval, download_job.http_code(), url.c_str());
             return BaseTN::kFailNetwork;
           }
           return BaseTN::kFailUnknown;
