@@ -7,8 +7,8 @@
 
 #include <string>
 
-#include "compression.h"
-#include "hash.h"
+#include "compression/compression.h"
+#include "crypto/hash.h"
 
 namespace receiver {
 
@@ -16,7 +16,9 @@ std::string GetSpoolerTempDir(const std::string& spooler_config);
 
 struct Params {
   std::string stratum0;
+  std::string proxy;
   std::string spooler_configuration;
+  bool use_local_cache;
   shash::Algorithms hash_alg;
   std::string hash_alg_str;
   zlib::Algorithms compression_alg;
@@ -33,6 +35,7 @@ struct Params {
   bool use_autocatalogs;
   size_t max_weight;
   size_t min_weight;
+  bool upload_stats_db;
 };
 
 bool GetParamsFromFile(const std::string& repo_name, Params* params);
