@@ -1269,7 +1269,7 @@ Supported Commands:
                   <fully qualified name>
                   Synchronize a Stratum 1 replica with the Stratum 0 source
   snapshot -a     [-s use separate logs in /var/log/cvmfs for each repository]
-                  [-n do not warn if /etc/logrotate.d/cvmfs does not exist]
+                  [-n DEPRECATED (used to warn if no logrotate config, now has no effect) ]
                   [-i skip repositories that have not run initial snapshot]
                   [-g group (do only the repositories in snapshot group)]
                   Do snapshot on all active replica repositories
@@ -1304,6 +1304,13 @@ Supported Commands:
   merge-stats     [-o output db file]
                   <db_file_1> <db_file_2>
                   Merge tables from two database files.
+  enter           [-w stratum0 url] [-c path to cvmfs2 binary] 
+                  [-C path to extra client config] [-r run as root]
+                  [-k keep session after shell exits] [-t open a transaction] 
+                  [-l remove session but keep logs after shell exits] 
+                  [-x path to repository configuration]
+                  <fully qualified repository name> [-- <command> <params>]
+                  Open an ephemeral namespace to publish content
 "
 
 
