@@ -7,16 +7,16 @@
  */
 
 #include "swissknife.h"
-#include "cvmfs_config.h"
+
 
 #include <unistd.h>
 
 #include <cassert>
 #include <vector>
 
-#include "logging.h"
 #include "manifest.h"
 #include "manifest_fetch.h"
+#include "util/logging.h"
 
 using namespace std;  // NOLINT
 
