@@ -2,7 +2,7 @@
  * This file is part of the CernVM File System.
  */
 
-#include "cvmfs_config.h"
+
 #include "swissknife_capabilities.h"
 
 #include <errno.h>
@@ -10,7 +10,7 @@
 
 #include <cassert>
 
-#include "logging.h"
+#include "util/logging.h"
 
 namespace swissknife {
 
