@@ -3,17 +3,17 @@
  */
 
 #include "swissknife_history.h"
-#include "cvmfs_config.h"
+
 
 #include <algorithm>
 #include <cassert>
 #include <ctime>
 
 #include "catalog_rw.h"
-#include "download.h"
-#include "hash.h"
+#include "crypto/hash.h"
+#include "crypto/signature.h"
 #include "manifest_fetch.h"
-#include "signature.h"
+#include "network/download.h"
 #include "upload.h"
 
 using namespace std;         // NOLINT
@@ -30,7 +30,6 @@ static void InsertCommonParameters(ParameterList *r) {
   r->push_back(Parameter::Mandatory('w', "repository directory / url"));
   r->push_back(Parameter::Mandatory('t', "temporary scratch directory"));
   r->push_back(Parameter::Optional('p', "public key of the repository"));
-  r->push_back(Parameter::Optional('z', "trusted certificates"));
   r->push_back(Parameter::Optional('f', "fully qualified repository name"));
   r->push_back(Parameter::Optional('r', "spooler definition string"));
   r->push_back(Parameter::Optional('m', "(unsigned) manifest file to edit"));
@@ -39,6 +38,7 @@ static void InsertCommonParameters(ParameterList *r) {
       Parameter::Optional('e', "hash algorithm to use (default SHA1)"));
   r->push_back(Parameter::Switch('L', "follow HTTP redirects"));
   r->push_back(Parameter::Optional('P', "session_token_file"));
+  r->push_back(Parameter::Optional('@', "proxy url"));
 }
 
 CommandTag::Environment *CommandTag::InitializeEnvironment(
@@ -59,9 +59,6 @@ CommandTag::Environment *CommandTag::InitializeEnvironment(
   const string pubkey_path = (args.find('p') == args.end())
                                  ? ""
                                  : MakeCanonicalPath(*args.find('p')->second);
-  const string trusted_certs = (args.find('z') == args.end())
-                                   ? ""
-                                   : MakeCanonicalPath(*args.find('z')->second);
   const shash::Any base_hash =
       (args.find('b') == args.end())
           ? shash::Any()
@@ -120,13 +117,14 @@ CommandTag::Environment *CommandTag::InitializeEnvironment(
 
   // initialize the (swissknife global) download manager
   const bool follow_redirects = (args.count('L') > 0);
-  if (!this->InitDownloadManager(follow_redirects)) {
+  const std::string &proxy = (args.count('@') > 0) ?
+                              *args.find('@')->second : "";
+  if (!this->InitDownloadManager(follow_redirects, proxy)) {
     return NULL;
   }
 
   // initialize the (swissknife global) signature manager (if possible)
-  if (!pubkey_path.empty() &&
-      !this->InitVerifyingSignatureManager(pubkey_path, trusted_certs)) {
+  if (!pubkey_path.empty() && !this->InitSignatureManager(pubkey_path)) {
     return NULL;
   }
 
@@ -137,7 +135,7 @@ CommandTag::Environment *CommandTag::InitializeEnvironment(
           ? OpenLocalManifest(env->manifest_path.path())
           : FetchRemoteManifest(env->repository_url, repo_name, base_hash);
 
-  if (!env->manifest) {
+  if (!env->manifest.IsValid()) {
     LogCvmfs(kLogCvmfs, kLogStderr, "failed to load manifest file");
     return NULL;
   }
@@ -146,7 +144,7 @@ CommandTag::Environment *CommandTag::InitializeEnvironment(
   if (read_write && env->manifest->history().IsNull() && !base_hash.IsNull()) {
     env->previous_manifest =
         FetchRemoteManifest(env->repository_url, repo_name, base_hash);
-    if (!env->previous_manifest) {
+    if (!env->previous_manifest.IsValid()) {
       LogCvmfs(kLogCvmfs, kLogStderr, "failed to load previous manifest");
       return NULL;
     }
@@ -164,7 +162,7 @@ CommandTag::Environment *CommandTag::InitializeEnvironment(
   // download the history database referenced in the manifest
   env->history = GetHistory(env->manifest.weak_ref(), env->repository_url,
                             env->history_path.path(), read_write);
-  if (!env->history) {
+  if (!env->history.IsValid()) {
     return NULL;
   }
 
@@ -180,7 +178,7 @@ CommandTag::Environment *CommandTag::InitializeEnvironment(
                                        use_file_chunking, 0, 0, 0,
                                        session_token_file);
     env->spooler = upload::Spooler::Construct(sd);
-    if (!env->spooler) {
+    if (!env->spooler.IsValid()) {
       LogCvmfs(kLogCvmfs, kLogStderr, "failed to initialize upload spooler");
       return NULL;
     }
@@ -225,8 +223,7 @@ bool CommandTag::CloseAndPublishHistory(Environment *env) {
   // disable the unlink guard in order to keep the newly exported manifest file
   env->manifest_path.Disable();
   LogCvmfs(kLogCvmfs, kLogVerboseMsg,
-           "exported manifest (%d) with new "
-           "history '%s'",
+           "exported manifest (%" PRIu64 ") with new history '%s'",
            env->manifest->revision(), new_history_hash.ToString().c_str());
 
   return true;
@@ -269,7 +266,7 @@ bool CommandTag::UploadCatalogAndUpdateManifest(
   env->manifest->set_catalog_size(catalog_size);
   env->manifest->set_catalog_hash(new_catalog_hash);
 
-  LogCvmfs(kLogCvmfs, kLogVerboseMsg, "uploaded new catalog (%d bytes) '%s'",
+  LogCvmfs(kLogCvmfs, kLogVerboseMsg, "uploaded new catalog (%lu bytes) '%s'",
            catalog_size, new_catalog_hash.ToString().c_str());
 
   return true;
@@ -313,7 +310,6 @@ bool CommandTag::UpdateUndoTags(
     if (!undo_rollback) {
       current_old_head = current_head;
       current_old_head.name = CommandTag::kPreviousHeadTag;
-      current_old_head.channel = history::History::kChannelTrunk;
       current_old_head.description = CommandTag::kPreviousHeadTagDescription;
       if (!env->history->Insert(current_old_head)) {
         LogCvmfs(kLogCvmfs, kLogStderr, "failed to set previous HEAD tag");
@@ -325,7 +321,6 @@ bool CommandTag::UpdateUndoTags(
   // set the current HEAD to the catalog provided by the template HEAD
   current_head = current_head_template;
   current_head.name = CommandTag::kHeadTag;
-  current_head.channel = history::History::kChannelTrunk;
   current_head.description = CommandTag::kHeadTagDescription;
   if (!env->history->Insert(current_head)) {
     LogCvmfs(kLogCvmfs, kLogStderr, "failed to set new current HEAD");
@@ -343,8 +338,8 @@ bool CommandTag::FetchObject(const std::string &repository_url,
   download::Failures dl_retval;
   const std::string url = repository_url + "/data/" + object_hash.MakePath();
 
-  download::JobInfo download_object(&url, true, false, &destination_path,
-                                    &object_hash);
+  cvmfs::PathSink pathsink(destination_path);
+  download::JobInfo download_object(&url, true, false, &object_hash, &pathsink);
   dl_retval = download_manager()->Fetch(&download_object);
 
   if (dl_retval != download::kFailOk) {
@@ -409,9 +404,10 @@ catalog::Catalog *CommandTag::GetCatalog(const std::string &repository_url,
 
 void CommandTag::PrintTagMachineReadable(
     const history::History::Tag &tag) const {
-  LogCvmfs(kLogCvmfs, kLogStdout, "%s %s %d %d %d %s %s %s", tag.name.c_str(),
+  LogCvmfs(kLogCvmfs, kLogStdout, "%s %s %" PRIu64 " %" PRIu64 " %ld %s %s",
+           tag.name.c_str(),
            tag.root_hash.ToString().c_str(), tag.size, tag.revision,
-           tag.timestamp, tag.GetChannelName(),
+           tag.timestamp,
            (tag.branch == "") ? "(default)" : tag.branch.c_str(),
            tag.description.c_str());
 }
@@ -445,7 +441,6 @@ ParameterList CommandEditTag::GetParams() const {
   r.push_back(Parameter::Optional('B', "branch of the new tag"));
   r.push_back(Parameter::Optional('P', "predecessor branch"));
   r.push_back(Parameter::Optional('h', "root hash of the new tag"));
-  r.push_back(Parameter::Optional('c', "channel of the new tag"));
   r.push_back(Parameter::Switch('x', "maintain undo tags"));
   return r;
 }
@@ -460,7 +455,7 @@ int CommandEditTag::Main(const ArgumentList &args) {
   // initialize the Environment (taking ownership)
   const bool history_read_write = true;
   UniquePtr<Environment> env(InitializeEnvironment(args, history_read_write));
-  if (!env) {
+  if (!env.IsValid()) {
     LogCvmfs(kLogCvmfs, kLogStderr, "failed to init environment");
     return 1;
   }
@@ -483,15 +478,10 @@ int CommandEditTag::Main(const ArgumentList &args) {
 }
 
 int CommandEditTag::AddNewTag(const ArgumentList &args, Environment *env) {
-  typedef history::History::UpdateChannel TagChannel;
   const std::string tag_name =
       (args.find('a') != args.end()) ? *args.find('a')->second : "";
   const std::string tag_description =
       (args.find('D') != args.end()) ? *args.find('D')->second : "";
-  const TagChannel tag_channel =
-      (args.find('c') != args.end())
-          ? static_cast<TagChannel>(String2Uint64(*args.find('c')->second))
-          : history::History::kChannelTrunk;
   const bool undo_tags = (args.find('x') != args.end());
   const std::string root_hash_string =
       (args.find('h') != args.end()) ? *args.find('h')->second : "";
@@ -519,13 +509,13 @@ int CommandEditTag::AddNewTag(const ArgumentList &args, Environment *env) {
     return 1;
   }
 
-  // open the catalog to be tagged (to check for existance and for meta info)
+  // open the catalog to be tagged (to check for existence and for meta info)
   const UnlinkGuard catalog_path(
       CreateTempPath(env->tmp_path + "/catalog", 0600));
   const bool catalog_read_write = false;
   const UniquePtr<catalog::Catalog> catalog(GetCatalog(
       env->repository_url, root_hash, catalog_path.path(), catalog_read_write));
-  if (!catalog) {
+  if (!catalog.IsValid()) {
     LogCvmfs(kLogCvmfs, kLogStderr, "catalog with hash '%s' does not exist",
              root_hash.ToString().c_str());
     return 1;
@@ -548,7 +538,6 @@ int CommandEditTag::AddNewTag(const ArgumentList &args, Environment *env) {
   tag_template.revision = catalog->GetRevision();
   tag_template.timestamp = catalog->GetLastModified();
   tag_template.branch = branch_name;
-  tag_template.channel = tag_channel;
   tag_template.description = tag_description;
 
   // manipulate the tag database by creating a new tag or moving an existing one
@@ -648,13 +637,6 @@ bool CommandEditTag::MoveTag(Environment *env,
     return false;
   }
 
-  // check that tag is not moved to another channel
-  if (new_tag.channel != old_tag.channel) {
-    LogCvmfs(kLogCvmfs, kLogStderr, "cannot move tag '%s' to another channel",
-             tag_name.c_str());
-    return false;
-  }
-
   // copy over old description if no new description was given
   if (new_tag.description.empty()) {
     new_tag.description = old_tag.description;
@@ -710,7 +692,7 @@ int CommandEditTag::RemoveTags(const ArgumentList &args, Environment *env) {
     }
   }
 
-  LogCvmfs(kLogCvmfs, kLogDebug, "proceeding to delete %d tags",
+  LogCvmfs(kLogCvmfs, kLogDebug, "proceeding to delete %lu tags",
            condemned_tags.size());
 
   // check if the tags to be deleted exist
@@ -773,7 +755,6 @@ void CommandListTags::PrintHumanReadableTagList(
   // go through the list of tags and figure out the column widths
   const std::string name_label = "Name";
   const std::string rev_label = "Revision";
-  const std::string chan_label = "Channel";
   const std::string time_label = "Timestamp";
   const std::string branch_label = "Branch";
   const std::string desc_label = "Description";
@@ -783,13 +764,11 @@ void CommandListTags::PrintHumanReadableTagList(
   const TagList::const_reverse_iterator iend = tags.rend();
   size_t max_name_len = name_label.size();
   size_t max_rev_len = rev_label.size();
-  size_t max_chan_len = chan_label.size();
   size_t max_time_len = desc_label.size();
   size_t max_branch_len = branch_label.size();
   for (; i != iend; ++i) {
     max_name_len = std::max(max_name_len, i->name.size());
     max_rev_len = std::max(max_rev_len, StringifyInt(i->revision).size());
-    max_chan_len = std::max(max_chan_len, strlen(i->GetChannelName()));
     max_time_len =
         std::max(max_time_len, StringifyTime(i->timestamp, true).size());
     max_branch_len = std::max(max_branch_len, i->branch.size());
@@ -797,19 +776,17 @@ void CommandListTags::PrintHumanReadableTagList(
 
   // print the list header
   LogCvmfs(kLogCvmfs, kLogStdout,
-           "%s \u2502 %s \u2502 %s \u2502 %s \u2502 %s \u2502 %s",
+           "%s \u2502 %s \u2502 %s \u2502 %s \u2502 %s",
            AddPadding(name_label, max_name_len).c_str(),
            AddPadding(rev_label, max_rev_len).c_str(),
-           AddPadding(chan_label, max_chan_len).c_str(),
            AddPadding(time_label, max_time_len).c_str(),
            AddPadding(branch_label, max_branch_len).c_str(),
            desc_label.c_str());
   LogCvmfs(kLogCvmfs, kLogStdout,
            "%s\u2500\u253C\u2500%s\u2500\u253C\u2500%s"
-           "\u2500\u253C\u2500%s\u2500\u253C\u2500%s\u2500\u253C\u2500%s",
+           "\u2500\u253C\u2500%s\u2500\u253C\u2500%s",
            AddPadding("", max_name_len, false, "\u2500").c_str(),
            AddPadding("", max_rev_len, false, "\u2500").c_str(),
-           AddPadding("", max_chan_len, false, "\u2500").c_str(),
            AddPadding("", max_time_len, false, "\u2500").c_str(),
            AddPadding("", max_branch_len, false, "\u2500").c_str(),
            AddPadding("", desc_label.size() + 1, false, "\u2500").c_str());
@@ -819,10 +796,9 @@ void CommandListTags::PrintHumanReadableTagList(
   for (; i != iend; ++i) {
     LogCvmfs(
         kLogCvmfs, kLogStdout,
-        "%s \u2502 %s \u2502 %s \u2502 %s \u2502 %s \u2502 %s",
+        "%s \u2502 %s \u2502 %s \u2502 %s \u2502 %s",
         AddPadding(i->name, max_name_len).c_str(),
         AddPadding(StringifyInt(i->revision), max_rev_len, true).c_str(),
-        AddPadding(i->GetChannelName(), max_chan_len).c_str(),
         AddPadding(StringifyTime(i->timestamp, true), max_time_len).c_str(),
         AddPadding(i->branch, max_branch_len).c_str(),
         i->description.c_str());
@@ -831,16 +807,15 @@ void CommandListTags::PrintHumanReadableTagList(
   // print the list footer
   LogCvmfs(kLogCvmfs, kLogStdout,
            "%s\u2500\u2534\u2500%s\u2500\u2534\u2500%s"
-           "\u2500\u2534\u2500%s\u2500\u2534\u2500%s\u2500\u2534\u2500%s",
+           "\u2500\u2534\u2500%s\u2500\u2534\u2500%s",
            AddPadding("", max_name_len, false, "\u2500").c_str(),
            AddPadding("", max_rev_len, false, "\u2500").c_str(),
-           AddPadding("", max_chan_len, false, "\u2500").c_str(),
            AddPadding("", max_time_len, false, "\u2500").c_str(),
            AddPadding("", max_branch_len, false, "\u2500").c_str(),
            AddPadding("", desc_label.size() + 1, false, "\u2500").c_str());
 
   // print the number of tags listed
-  LogCvmfs(kLogCvmfs, kLogStdout, "listing contains %d tags", tags.size());
+  LogCvmfs(kLogCvmfs, kLogStdout, "listing contains %lu tags", tags.size());
 }
 
 void CommandListTags::PrintMachineReadableTagList(const TagList &tags) const {
@@ -861,7 +836,7 @@ void CommandListTags::PrintHumanReadableBranchList(
       LogCvmfs(kLogCvmfs, kLogStdout | kLogNoLinebreak, "%s",
                ((l + 1) == branches[i].level) ? "\u251c " : "\u2502 ");
     }
-    LogCvmfs(kLogCvmfs, kLogStdout, "%s @%u",
+    LogCvmfs(kLogCvmfs, kLogStdout, "%s @%" PRIu64,
              branches[i].branch.branch.c_str(),
              branches[i].branch.initial_revision);
   }
@@ -873,7 +848,7 @@ void CommandListTags::PrintMachineReadableBranchList(
 {
   unsigned N = branches.size();
   for (unsigned i = 0; i < N; ++i) {
-    LogCvmfs(kLogCvmfs, kLogStdout, "[%u] %s%s @%u",
+    LogCvmfs(kLogCvmfs, kLogStdout, "[%u] %s%s @%" PRIu64,
              branches[i].level,
              AddPadding("", branches[i].level, false, " ").c_str(),
              branches[i].branch.branch.c_str(),
@@ -921,7 +896,7 @@ int CommandListTags::Main(const ArgumentList &args) {
   // initialize the Environment (taking ownership)
   const bool history_read_write = false;
   UniquePtr<Environment> env(InitializeEnvironment(args, history_read_write));
-  if (!env) {
+  if (!env.IsValid()) {
     LogCvmfs(kLogCvmfs, kLogStderr, "failed to init environment");
     return 1;
   }
@@ -990,14 +965,13 @@ void CommandInfoTag::PrintHumanReadableInfo(
     const history::History::Tag &tag) const {
   LogCvmfs(kLogCvmfs, kLogStdout,
            "Name:         %s\n"
-           "Revision:     %d\n"
-           "Channel:      %s\n"
+           "Revision:     %" PRIu64 "\n"
            "Timestamp:    %s\n"
            "Branch:       %s\n"
            "Root Hash:    %s\n"
            "Catalog Size: %s\n"
            "%s",
-           tag.name.c_str(), tag.revision, tag.GetChannelName(),
+           tag.name.c_str(), tag.revision,
            StringifyTime(tag.timestamp, true /* utc */).c_str(),
            tag.branch.c_str(),
            tag.root_hash.ToString().c_str(),
@@ -1012,7 +986,7 @@ int CommandInfoTag::Main(const ArgumentList &args) {
   // initialize the Environment (taking ownership)
   const bool history_read_write = false;
   UniquePtr<Environment> env(InitializeEnvironment(args, history_read_write));
-  if (!env) {
+  if (!env.IsValid()) {
     LogCvmfs(kLogCvmfs, kLogStderr, "failed to init environment");
     return 1;
   }
@@ -1052,7 +1026,7 @@ int CommandRollbackTag::Main(const ArgumentList &args) {
   // initialize the Environment (taking ownership)
   const bool history_read_write = true;
   UniquePtr<Environment> env(InitializeEnvironment(args, history_read_write));
-  if (!env) {
+  if (!env.IsValid()) {
     LogCvmfs(kLogCvmfs, kLogStderr, "failed to init environment");
     return 1;
   }
@@ -1090,7 +1064,8 @@ int CommandRollbackTag::Main(const ArgumentList &args) {
   const uint64_t current_revision = env->manifest->revision();
   assert(target_tag.revision <= current_revision);
   if (target_tag.revision == current_revision) {
-    LogCvmfs(kLogCvmfs, kLogStderr, "not rolling back to current head (%u)",
+    LogCvmfs(kLogCvmfs, kLogStderr,
+             "not rolling back to current head (%" PRIu64 ")",
              current_revision);
     return 1;
   }
@@ -1103,7 +1078,7 @@ int CommandRollbackTag::Main(const ArgumentList &args) {
       dynamic_cast<catalog::WritableCatalog *>(
           GetCatalog(env->repository_url, target_tag.root_hash,
                      catalog_path.path(), catalog_read_write)));
-  if (!catalog) {
+  if (!catalog.IsValid()) {
     LogCvmfs(kLogCvmfs, kLogStderr, "failed to open catalog with hash '%s'",
              target_tag.root_hash.ToString().c_str());
     return 1;
@@ -1192,7 +1167,7 @@ int CommandEmptyRecycleBin::Main(const ArgumentList &args) {
   // initialize the Environment (taking ownership)
   const bool history_read_write = true;
   UniquePtr<Environment> env(InitializeEnvironment(args, history_read_write));
-  if (!env) {
+  if (!env.IsValid()) {
     LogCvmfs(kLogCvmfs, kLogStderr, "failed to init environment");
     return 1;
   }
