@@ -13,11 +13,11 @@
 #include "catalog_rw.h"
 #include "catalog_sql.h"
 #include "catalog_virtual.h"
-#include "compression.h"
-#include "hash.h"
-#include "logging.h"
+#include "compression/compression.h"
+#include "crypto/hash.h"
 #include "swissknife_history.h"
-#include "util_concurrency.h"
+#include "util/concurrency.h"
+#include "util/logging.h"
 
 using namespace std;  // NOLINT
 
@@ -40,7 +40,8 @@ CommandMigrate::CommandMigrate() :
 ParameterList CommandMigrate::GetParams() const {
   ParameterList r;
   r.push_back(Parameter::Mandatory('v',
-    "migration base version ( 2.0.x | 2.1.7 | chown | hardlink )"));
+    "migration base version ( 2.0.x | 2.1.7 | chown | hardlink | bulkhash | "
+    "stats)"));
   r.push_back(Parameter::Mandatory('r',
     "repository URL (absolute local path or remote URL)"));
   r.push_back(Parameter::Mandatory('u', "upstream definition string"));
@@ -56,6 +57,7 @@ ParameterList CommandMigrate::GetParams() const {
   r.push_back(Parameter::Optional('k', "repository master key(s)"));
   r.push_back(Parameter::Optional('i', "UID map for chown"));
   r.push_back(Parameter::Optional('j', "GID map for chown"));
+  r.push_back(Parameter::Optional('@', "proxy url"));
   r.push_back(Parameter::Switch('f', "fix nested catalog transition points"));
   r.push_back(Parameter::Switch('l', "disable linkcount analysis of files"));
   r.push_back(Parameter::Switch('s',
@@ -65,7 +67,7 @@ ParameterList CommandMigrate::GetParams() const {
 
 
 static void Error(const std::string &message) {
-  LogCvmfs(kLogCatalog, kLogStderr, message.c_str());
+  LogCvmfs(kLogCatalog, kLogStderr, "%s", message.c_str());
 }
 
 
@@ -137,7 +139,7 @@ int CommandMigrate::Main(const ArgumentList &args) {
   temporary_directory_ = tmp_dir;
   const upload::SpoolerDefinition spooler_definition(spooler, shash::kSha1);
   spooler_ = upload::Spooler::Construct(spooler_definition);
-  if (!spooler_) {
+  if (!spooler_.IsValid()) {
     Error("Failed to create upstream Spooler.");
     return 5;
   }
@@ -152,8 +154,9 @@ int CommandMigrate::Main(const ArgumentList &args) {
     typedef HttpObjectFetcher<catalog::WritableCatalog> ObjectFetcher;
 
     const bool follow_redirects = false;
-    if (!this->InitDownloadManager(follow_redirects) ||
-        !this->InitVerifyingSignatureManager(repo_keys)) {
+    const string proxy = (args.count('@') > 0) ? *args.find('@')->second : "";
+    if (!this->InitDownloadManager(follow_redirects, proxy) ||
+        !this->InitSignatureManager(repo_keys)) {
       LogCvmfs(kLogCatalog, kLogStderr, "Failed to init repo connection");
       return 1;
     }
@@ -227,6 +230,17 @@ int CommandMigrate::Main(const ArgumentList &args) {
     migration_succeeded =
       DoMigrationAndCommit<HardlinkRemovalMigrationWorker>(manifest_path,
                                                            &context);
+  } else if (migration_base == "bulkhash") {
+    BulkhashRemovalMigrationWorker::worker_context
+      context(temporary_directory_, collect_catalog_statistics);
+    migration_succeeded =
+      DoMigrationAndCommit<BulkhashRemovalMigrationWorker>(manifest_path,
+                                                           &context);
+  } else if (migration_base == "stats") {
+    StatsMigrationWorker::worker_context context(
+      temporary_directory_, collect_catalog_statistics);
+    migration_succeeded =
+      DoMigrationAndCommit<StatsMigrationWorker>(manifest_path, &context);
   } else {
     const std::string err_msg = "Unknown migration base: " + migration_base;
     Error(err_msg);
@@ -313,7 +327,7 @@ void CommandMigrate::UploadHistoryClosure(
 
 bool CommandMigrate::UpdateUndoTags(
   PendingCatalog *root_catalog,
-  unsigned revision,
+  uint64_t revision,
   time_t timestamp,
   shash::Any *history_hash)
 {
@@ -338,7 +352,6 @@ bool CommandMigrate::UpdateUndoTags(
 
     tag_trunk.root_hash = root_catalog->new_catalog_hash;
     tag_trunk.size = root_catalog->new_catalog_size;
-    tag_trunk.revision = root_catalog->new_catalog_size;
     tag_trunk.revision = revision;
     tag_trunk.timestamp = timestamp;
 
@@ -429,7 +442,7 @@ bool CommandMigrate::DoMigrationAndCommit(
                           new_catalog->GetLastModified(),
                           &history_hash))
       {
-        Error("Updateing tag database failed.\nAborting...");
+        Error("Updating tag database failed.\nAborting...");
         return false;
       }
       manifest.set_history(history_hash);
@@ -734,8 +747,8 @@ bool CommandMigrate::AbstractMigrationWorker<DerivedT>::
     "INSERT OR REPLACE INTO nested_catalogs (path,   sha1,  size) "
     "                VALUES                 (:path, :sha1, :size);");
 
-  // go through all nested catalogs and update their references (we are curently
-  // in their parent catalog)
+  // go through all nested catalogs and update their references (we are
+  // currently in their parent catalog)
   // Note: we might need to wait for the nested catalog to be fully processed.
   PendingCatalogList::const_iterator i    = data->nested_catalogs.begin();
   PendingCatalogList::const_iterator iend = data->nested_catalogs.end();
@@ -846,7 +859,7 @@ bool CommandMigrate::AbstractMigrationWorker<DerivedT>::CleanupNestedCatalogs(
  * both the catalog management and migration classes get updated.
  */
 const float    CommandMigrate::MigrationWorker_20x::kSchema         = 2.5;
-const unsigned CommandMigrate::MigrationWorker_20x::kSchemaRevision = 5;
+const unsigned CommandMigrate::MigrationWorker_20x::kSchemaRevision = 7;
 
 
 template<class DerivedT>
@@ -858,6 +871,9 @@ CommandMigrate::AbstractMigrationWorker<DerivedT>::GetWritable(
 }
 
 
+//------------------------------------------------------------------------------
+
+
 CommandMigrate::MigrationWorker_20x::MigrationWorker_20x(
   const worker_context *context)
   : AbstractMigrationWorker<MigrationWorker_20x>(context)
@@ -1090,7 +1106,7 @@ bool CommandMigrate::MigrationWorker_20x::MigrateFileMetadata(
     "         IFNULL(hardlink_group_id, 0) << 32 | "
     "         COALESCE(hardlinks.linkcount, dir_linkcounts.linkcount, 1) "
     "           AS hardlinks, "
-    "         hash, size, mode, mtime, "
+    "         hash, size, mode, mtime, NULL, " // set empty mtimens
     "         flags, name, symlink, "
     "         :uid, "
     "         :gid, "
@@ -1393,7 +1409,7 @@ bool CommandMigrate::MigrationWorker_20x::FixNestedCatalogTransitionPoints(
       update_directory_entry.Reset();
 
       // Fixing of this mountpoint went well... inform the user that this minor
-      // issue occured
+      // issue occurred
       LogCvmfs(kLogCatalog, kLogStdout,
                "NOTE: fixed incompatible nested catalog transition point at: "
                "'%s' ", nested_root_path.c_str());
@@ -1544,7 +1560,8 @@ bool CommandMigrate::MigrationWorker_20x::GenerateCatalogStatistics(
   const catalog::CatalogDatabase &writable = data->new_catalog->database();
 
   // Aggregated the statistics counters of all nested catalogs
-  // Note: we might need to wait until nested catalogs are sucessfully processed
+  // Note: we might need to wait until nested catalogs are successfully
+  // processed
   catalog::DeltaCounters stats_counters;
   PendingCatalogList::const_iterator i    = data->nested_catalogs.begin();
   PendingCatalogList::const_iterator iend = data->nested_catalogs.end();
@@ -1728,7 +1745,8 @@ bool CommandMigrate::MigrationWorker_217::GenerateNewStatisticsCounters
     GetWritable(data->old_catalog)->database();
 
   // Aggregated the statistics counters of all nested catalogs
-  // Note: we might need to wait until nested catalogs are sucessfully processed
+  // Note: we might need to wait until nested catalogs are successfully
+  // processed
   catalog::DeltaCounters stats_counters;
   PendingCatalogList::const_iterator i = data->nested_catalogs.begin();
   PendingCatalogList::const_iterator iend = data->nested_catalogs.end();
@@ -1979,4 +1997,200 @@ bool CommandMigrate::HardlinkRemovalMigrationWorker::BreakUpHardlinks(
   return db.CommitTransaction();
 }
 
+//------------------------------------------------------------------------------
+
+
+bool CommandMigrate::BulkhashRemovalMigrationWorker::RunMigration(
+                                                   PendingCatalog *data) const {
+  return CheckDatabaseSchemaCompatibility(data) &&
+         RemoveRedundantBulkHashes(data);
+}
+
+
+bool
+CommandMigrate::BulkhashRemovalMigrationWorker::CheckDatabaseSchemaCompatibility
+                                                  (PendingCatalog *data) const {
+  assert(data->old_catalog != NULL);
+  assert(data->new_catalog == NULL);
+
+  const catalog::CatalogDatabase &clg = data->old_catalog->database();
+  return clg.schema_version() >= 2.4 - catalog::CatalogDatabase::kSchemaEpsilon;
+}
+
+
+bool CommandMigrate::BulkhashRemovalMigrationWorker::RemoveRedundantBulkHashes(
+                                                   PendingCatalog *data) const {
+  assert(data->old_catalog != NULL);
+  assert(data->new_catalog == NULL);
+
+  const catalog::CatalogDatabase &db =
+                                     GetWritable(data->old_catalog)->database();
+
+  if (!db.BeginTransaction()) {
+    return false;
+  }
+
+  // Regular files with both bulk hashes and chunked hashes can drop the bulk
+  // hash since modern clients >= 2.1.7 won't require them
+  const std::string stmt = "UPDATE OR ABORT catalog "
+                           "SET hash = NULL "
+                           "WHERE flags & :file_chunked_flag;";
+  catalog::SqlCatalog bulkhash_removal_sql(db, stmt);
+  bulkhash_removal_sql.BindInt64(1, catalog::SqlDirent::kFlagFileChunk);
+  bulkhash_removal_sql.Execute();
+
+  return db.CommitTransaction();
+}
+
+
+//------------------------------------------------------------------------------
+
+
+CommandMigrate::StatsMigrationWorker::StatsMigrationWorker(
+  const worker_context *context)
+  : AbstractMigrationWorker<StatsMigrationWorker>(context)
+{ }
+
+
+bool CommandMigrate::StatsMigrationWorker::RunMigration(PendingCatalog *data)
+  const
+{
+  return CheckDatabaseSchemaCompatibility(data) &&
+         StartDatabaseTransaction(data) &&
+         RepairStatisticsCounters(data) &&
+         CommitDatabaseTransaction(data);
+}
+
+
+bool CommandMigrate::StatsMigrationWorker::CheckDatabaseSchemaCompatibility(
+  PendingCatalog *data) const
+{
+  assert(data->old_catalog != NULL);
+  assert(data->new_catalog == NULL);
+
+  const catalog::CatalogDatabase &clg = data->old_catalog->database();
+  if (clg.schema_version() < 2.5 - catalog::CatalogDatabase::kSchemaEpsilon) {
+    Error("Given catalog schema is < 2.5.", data);
+    return false;
+  }
+
+  if (clg.schema_revision() < 5) {
+    Error("Given catalog revision is < 5", data);
+    return false;
+  }
+
+  return true;
+}
+
+
+bool CommandMigrate::StatsMigrationWorker::StartDatabaseTransaction(
+  PendingCatalog *data) const
+{
+  assert(!data->HasNew());
+  GetWritable(data->old_catalog)->Transaction();
+  return true;
+}
+
+
+bool CommandMigrate::StatsMigrationWorker::RepairStatisticsCounters(
+  PendingCatalog *data) const
+{
+  assert(!data->HasNew());
+  bool retval = false;
+  const catalog::CatalogDatabase &writable =
+    GetWritable(data->old_catalog)->database();
+
+  // Aggregated the statistics counters of all nested catalogs
+  // Note: we might need to wait until nested catalogs are successfully
+  // processed
+  catalog::DeltaCounters stats_counters;
+  PendingCatalogList::const_iterator i = data->nested_catalogs.begin();
+  PendingCatalogList::const_iterator iend = data->nested_catalogs.end();
+  for (; i != iend; ++i) {
+    const PendingCatalog *nested_catalog = *i;
+    const catalog::DeltaCounters &s = nested_catalog->nested_statistics.Get();
+    s.PopulateToParent(&stats_counters);
+  }
+
+  // Count various directory entry types in the catalog to fill up the catalog
+  // statistics counters introduced in the current catalog schema
+  catalog::SqlCatalog count_regular(writable,
+    std::string("SELECT count(*), sum(size) FROM catalog ") +
+    "WHERE flags & " + StringifyInt(catalog::SqlDirent::kFlagFile) +
+    " AND NOT flags & " + StringifyInt(catalog::SqlDirent::kFlagLink) +
+    " AND NOT flags & " + StringifyInt(catalog::SqlDirent::kFlagFileSpecial) +
+    ";");
+  catalog::SqlCatalog count_external(writable,
+    std::string("SELECT count(*), sum(size) FROM catalog ") +
+    "WHERE flags & " + StringifyInt(catalog::SqlDirent::kFlagFileExternal) +
+    ";");
+  catalog::SqlCatalog count_symlink(writable,
+    std::string("SELECT count(*) FROM catalog ") +
+    "WHERE flags & " + StringifyInt(catalog::SqlDirent::kFlagLink) + ";");
+  catalog::SqlCatalog count_special(writable,
+    std::string("SELECT count(*) FROM catalog ") +
+    "WHERE flags & " + StringifyInt(catalog::SqlDirent::kFlagFileSpecial) +
+    ";");
+  catalog::SqlCatalog count_xattr(writable,
+    std::string("SELECT count(*) FROM catalog ") +
+    "WHERE xattr IS NOT NULL;");
+  catalog::SqlCatalog count_chunk(writable,
+    std::string("SELECT count(*), sum(size) FROM catalog ") +
+    "WHERE flags & " + StringifyInt(catalog::SqlDirent::kFlagFileChunk) + ";");
+  catalog::SqlCatalog count_dir(writable,
+    std::string("SELECT count(*) FROM catalog ") +
+    "WHERE flags & " + StringifyInt(catalog::SqlDirent::kFlagDir) + ";");
+  catalog::SqlCatalog count_chunk_blobs(writable,
+    "SELECT count(*) FROM chunks;");
+
+  retval = count_regular.FetchRow() &&
+           count_external.FetchRow() &&
+           count_symlink.FetchRow() &&
+           count_special.FetchRow() &&
+           count_xattr.FetchRow() &&
+           count_chunk.FetchRow() &&
+           count_dir.FetchRow() &&
+           count_chunk_blobs.FetchRow();
+  if (!retval) {
+    Error("Failed to collect catalog statistics", data);
+    return false;
+  }
+
+  stats_counters.self.regular_files       = count_regular.RetrieveInt64(0);
+  stats_counters.self.symlinks            = count_symlink.RetrieveInt64(0);
+  stats_counters.self.specials            = count_special.RetrieveInt64(0);
+  stats_counters.self.directories         = count_dir.RetrieveInt64(0);
+  stats_counters.self.nested_catalogs     = data->nested_catalogs.size();
+  stats_counters.self.chunked_files       = count_chunk.RetrieveInt64(0);
+  stats_counters.self.file_chunks         = count_chunk_blobs.RetrieveInt64(0);
+  stats_counters.self.file_size           = count_regular.RetrieveInt64(1);
+  stats_counters.self.chunked_file_size   = count_chunk.RetrieveInt64(1);
+  stats_counters.self.xattrs              = count_xattr.RetrieveInt64(0);
+  stats_counters.self.externals           = count_external.RetrieveInt64(0);
+  stats_counters.self.external_file_size  = count_external.RetrieveInt64(1);
+
+  // Write back the generated statistics counters into the catalog database
+  catalog::Counters counters;
+  counters.ApplyDelta(stats_counters);
+  retval = counters.InsertIntoDatabase(writable);
+  if (!retval) {
+    Error("Failed to write new statistics counters to database", data);
+    return false;
+  }
+
+  // Push the generated statistics counters up to the parent catalog
+  data->nested_statistics.Set(stats_counters);
+
+  return true;
+}
+
+
+bool CommandMigrate::StatsMigrationWorker::CommitDatabaseTransaction(
+  PendingCatalog *data) const
+{
+  assert(!data->HasNew());
+  GetWritable(data->old_catalog)->Commit();
+  return true;
+}
+
 }  // namespace swissknife
