@@ -34,7 +34,8 @@ class SyncUnionTarball : public SyncUnion {
                    const uid_t uid,
                    const gid_t gid,
                    const std::string &to_delete,
-                   const bool create_catalog_on_root);
+                   const bool create_catalog_on_root,
+                   const std::string &path_delimiter=":");
 
   ~SyncUnionTarball();
 
@@ -69,6 +70,7 @@ class SyncUnionTarball : public SyncUnion {
   const gid_t gid_;
   const std::string to_delete_;  ///< entity to delete before to extract the tar
   const bool create_catalog_on_root_;
+  const std::string path_delimiter_; ///< delimiter used to split paths
   std::set<std::string>
       know_directories_;  ///< directory that we know already exist
 
