@@ -5,10 +5,11 @@
  */
 
 #ifndef __STDC_FORMAT_MACROS
+// NOLINTNEXTLINE
 #define __STDC_FORMAT_MACROS
 #endif
 
-#include "cvmfs_config.h"
+
 #include "posix.h"
 
 #include <arpa/inet.h>
@@ -24,9 +25,15 @@
 #include <sys/resource.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
+#ifdef __APPLE__
+#include <sys/mount.h>  //  for statfs()
+#else
+#include <sys/statfs.h>
+#endif
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/un.h>
+#include <sys/utsname.h>
 #include <sys/wait.h>
 #include <unistd.h>
 // If valgrind headers are present on the build system, then we can detect
@@ -44,14 +51,40 @@
 #include <string>
 #include <vector>
 
-#include "fs_traversal.h"
-#include "logging.h"
-#include "platform.h"
-
-#include "util_concurrency.h"
+#include "util/algorithm.h"
+#include "util/concurrency.h"
+#include "util/exception.h"
+#include "util/fs_traversal.h"
+#include "util/logging.h"
+#include "util/pipe.h"
+#include "util/platform.h"
+#include "util/string.h"
 
 //using namespace std;  // NOLINT
 
+#ifndef ST_RDONLY
+// On Linux, this is in sys/statvfs.h
+// On macOS, this flag is called MNT_RDONLY /usr/include/sys/mount.h
+#define ST_RDONLY 1
+#endif
+
+// Older Linux glibc versions do not provide the f_flags member in struct statfs
+#define CVMFS_HAS_STATFS_F_FLAGS
+#ifndef __APPLE__
+#ifdef __GLIBC_MINOR__
+#if __GLIBC_MINOR__ < 12
+#undef CVMFS_HAS_STATFS_F_FLAGS
+#endif
+#endif
+#endif
+
+// Work around missing clearenv()
+#ifdef __APPLE__
+extern "C" {
+extern char **environ;
+}
+#endif
+
 #ifdef CVMFS_NAMESPACE_GUARD
 namespace CVMFS_NAMESPACE_GUARD {
 #endif
@@ -65,13 +98,13 @@ static pthread_mutex_t getumask_mutex = PTHREAD_MUTEX_INITIALIZER;
 std::string MakeCanonicalPath(const std::string &path) {
   if (path.length() == 0) return path;
 
-  if (path[path.length()-1] == '/')
+  if (path[path.length()-1] == '/') {
     return path.substr(0, path.length()-1);
-  else
+  } else {
     return path;
+  }
 }
 
-
 /**
  * Return both the file and directory name for a given path.
  *
@@ -98,28 +131,11 @@ void SplitPath(
  */
 std::string GetParentPath(const std::string &path) {
   const std::string::size_type idx = path.find_last_of('/');
-  if (idx != std::string::npos)
+  if (idx != std::string::npos) {
     return path.substr(0, idx);
-  else
+  } else {
     return "";
-}
-
-
-/**
- * Gets the file name part of a path.
- */
-PathString GetParentPath(const PathString &path) {
-  unsigned length = path.GetLength();
-  if (length == 0)
-    return path;
-  const char *chars  = path.GetChars();
-
-  for (int i = length-1; i >= 0; --i) {
-    if (chars[i] == '/')
-      return PathString(chars, i);
   }
-
-  return path;
 }
 
 
@@ -128,29 +144,11 @@ PathString GetParentPath(const PathString &path) {
  */
 std::string GetFileName(const std::string &path) {
   const std::string::size_type idx = path.find_last_of('/');
-  if (idx != std::string::npos)
+  if (idx != std::string::npos) {
     return path.substr(idx+1);
-  else
+  } else {
     return path;
-}
-
-
-NameString GetFileName(const PathString &path) {
-  NameString name;
-  int length = path.GetLength();
-  const char *chars  = path.GetChars();
-
-  int i;
-  for (i = length-1; i >= 0; --i) {
-    if (chars[i] == '/')
-      break;
   }
-  i++;
-  if (i < length) {
-    name.Append(chars+i, length-i);
-  }
-
-  return name;
 }
 
 
@@ -172,15 +170,112 @@ bool IsHttpUrl(const std::string &path) {
     return false;
   }
 
-  std::string prefix = path.substr(0, 7);
+  std::string prefix = path.substr(0, 8);
   std::transform(prefix.begin(), prefix.end(), prefix.begin(), ::tolower);
 
-  return prefix == "http://";
+  return prefix.substr(0, 7) == "http://" || prefix == "https://";
+}
+
+
+FileSystemInfo GetFileSystemInfo(const std::string &path) {
+  FileSystemInfo result;
+
+  struct statfs info;
+  int retval = statfs(path.c_str(), &info);
+  if (retval != 0)
+    return result;
+
+  switch (info.f_type) {
+    case kFsTypeAutofs:
+      result.type = kFsTypeAutofs;
+      break;
+    case kFsTypeNFS:
+      result.type = kFsTypeNFS;
+      break;
+    case kFsTypeProc:
+      result.type = kFsTypeProc;
+      break;
+    case kFsTypeBeeGFS:
+      result.type = kFsTypeBeeGFS;
+      break;
+    case kFsTypeTmpfs:
+      result.type = kFsTypeTmpfs;
+      break;
+    default:
+      result.type = kFsTypeUnknown;
+  }
+
+#ifdef CVMFS_HAS_STATFS_F_FLAGS
+  if (info.f_flags & ST_RDONLY)
+    result.is_rdonly = true;
+#else
+  // On old Linux systems, fall back to access()
+  retval = access(path.c_str(), W_OK);
+  result.is_rdonly = (retval != 0);
+#endif
+
+
+
+  return result;
+}
+
+
+std::string ReadSymlink(const std::string &path) {
+  // TODO(jblomer): avoid PATH_MAX
+  char buf[PATH_MAX + 1];
+  ssize_t nchars = readlink(path.c_str(), buf, PATH_MAX);
+  if (nchars >= 0) {
+    buf[nchars] = '\0';
+    return std::string(buf);
+  }
+  return "";
 }
 
 
 /**
- * By default abort() on failure
+ * Follow all symlinks if possible. Equivalent to
+ * `readlink --canonicalize-missing`
+ */
+std::string ResolvePath(const std::string &path) {
+  if (path.empty() || (path == "/"))
+    return "/";
+  std::string name = GetFileName(path);
+  std::string result = name;
+  if (name != path) {
+    // There is a parent path of 'path'
+    std::string parent = ResolvePath(GetParentPath(path));
+    result = parent + (parent == "/" ? "" : "/") + name;
+  }
+  char *real_result = realpath(result.c_str(), NULL);
+  if (real_result) {
+    result = real_result;
+    free(real_result);
+  }
+  if (SymlinkExists(result)) {
+    char buf[PATH_MAX + 1];
+    ssize_t nchars = readlink(result.c_str(), buf, PATH_MAX);
+    if (nchars >= 0) {
+      buf[nchars] = '\0';
+      result = buf;
+    }
+  }
+  return result;
+}
+
+
+bool IsMountPoint(const std::string &path) {
+  std::vector<std::string> mount_list = platform_mountlist();
+  std::string resolved_path = ResolvePath(path);
+  for (unsigned i = 0; i < mount_list.size(); ++i) {
+    if (mount_list[i] == resolved_path)
+      return true;
+  }
+  return false;
+}
+
+
+/**
+ * By default PANIC(NULL) on failure
  */
 void CreateFile(
   const std::string &path,
@@ -194,7 +289,7 @@ void CreateFile(
   }
   if (ignore_failure)
     return;
-  assert(false);
+  PANIC(NULL);
 }
 
 
@@ -257,12 +352,12 @@ int MakeSocket(const std::string &path, const int mode) {
     goto make_socket_failure;
 #endif
 
-  if (bind(socket_fd, (struct sockaddr *)&sock_addr,
+  if (bind(socket_fd, reinterpret_cast<struct sockaddr *>(&sock_addr),
            sizeof(sock_addr.sun_family) + sizeof(sock_addr.sun_path)) < 0)
   {
     if ((errno == EADDRINUSE) && (unlink(path.c_str()) == 0)) {
       // Second try, perhaps the file was left over
-      if (bind(socket_fd, (struct sockaddr *)&sock_addr,
+      if (bind(socket_fd, reinterpret_cast<struct sockaddr *>(&sock_addr),
                sizeof(sock_addr.sun_family) + sizeof(sock_addr.sun_path)) < 0)
       {
         LogCvmfs(kLogCvmfs, kLogDebug, "binding socket failed (%d)", errno);
@@ -313,7 +408,7 @@ int MakeTcpEndpoint(const std::string &ipv4_address, int portno) {
   }
   endpoint_addr.sin_port = htons(portno);
 
-  retval = bind(socket_fd, (struct sockaddr *)&endpoint_addr,
+  retval = bind(socket_fd, reinterpret_cast<struct sockaddr *>(&endpoint_addr),
                 sizeof(endpoint_addr));
   if (retval < 0) {
     LogCvmfs(kLogCvmfs, kLogDebug, "binding TCP endpoint failed (%d)", errno);
@@ -346,13 +441,12 @@ int ConnectSocket(const std::string &path) {
   assert(socket_fd != -1);
 
   int retval =
-    connect(socket_fd, (struct sockaddr *)&sock_addr,
+    connect(socket_fd, reinterpret_cast<struct sockaddr *>(&sock_addr),
             sizeof(sock_addr.sun_family) + sizeof(sock_addr.sun_path));
   if (short_path != path)
     RemoveShortSocketLink(short_path);
 
   if (retval < 0) {
-    // LogCvmfs(kLogCvmfs, kLogStderr, "ERROR %d", errno);
     close(socket_fd);
     return -1;
   }
@@ -379,8 +473,9 @@ int ConnectTcpEndpoint(const std::string &ipv4_address, int portno) {
   }
   endpoint_addr.sin_port = htons(portno);
 
-  retval = connect(socket_fd, (struct sockaddr *)&endpoint_addr,
-                   sizeof(endpoint_addr));
+  retval =
+  connect(socket_fd, reinterpret_cast<struct sockaddr *>(&endpoint_addr),
+          sizeof(endpoint_addr));
   if (retval != 0) {
     LogCvmfs(kLogCvmfs, kLogDebug, "failed to connect to TCP endpoint (%d)",
              errno);
@@ -404,7 +499,7 @@ void MakePipe(int pipe_fd[2]) {
  * Writes to a pipe should always succeed.
  */
 void WritePipe(int fd, const void *buf, size_t nbyte) {
-  int num_bytes;
+  ssize_t num_bytes;
   do {
     num_bytes = write(fd, buf, nbyte);
   } while ((num_bytes < 0) && (errno == EINTR));
@@ -416,7 +511,7 @@ void WritePipe(int fd, const void *buf, size_t nbyte) {
  * Reads from a pipe should always succeed.
  */
 void ReadPipe(int fd, void *buf, size_t nbyte) {
-  int num_bytes;
+  ssize_t num_bytes;
   do {
     num_bytes = read(fd, buf, nbyte);
   } while ((num_bytes < 0) && (errno == EINTR));
@@ -427,10 +522,15 @@ void ReadPipe(int fd, void *buf, size_t nbyte) {
 /**
  * Reads from a pipe where writer's end is not yet necessarily connected
  */
-void ReadHalfPipe(int fd, void *buf, size_t nbyte) {
-  int num_bytes;
+bool ReadHalfPipe(int fd, void *buf, size_t nbyte, unsigned timeout_ms) {
+  ssize_t num_bytes;
   unsigned i = 0;
   unsigned backoff_ms = 1;
+  uint64_t duration_ms = 0;
+  uint64_t timestamp = 0;
+  if (timeout_ms != 0)
+    timestamp = platform_monotonic_time_ns();
+
   const unsigned max_backoff_ms = 256;
   do {
     // When the writer is not connected, this takes ~200-300ns per call as per
@@ -445,8 +545,14 @@ void ReadHalfPipe(int fd, void *buf, size_t nbyte) {
       SafeSleepMs(backoff_ms);
       if (backoff_ms < max_backoff_ms) backoff_ms *= 2;
     }
+    if ((timeout_ms != 0) && (num_bytes == 0)) {
+      duration_ms = (platform_monotonic_time_ns() - timestamp) / (1000UL * 1000UL);
+      if (duration_ms  > timeout_ms)
+        return false;
+    }
   } while (num_bytes == 0);
   assert((num_bytes >= 0) && (static_cast<size_t>(num_bytes) == nbyte));
+  return true;
 }
 
 
@@ -519,7 +625,7 @@ bool DiffTree(const std::string &path_a, const std::string &path_b) {
     if ((info_a.st_mode != info_b.st_mode) ||
         (info_a.st_uid != info_b.st_uid) ||
         (info_a.st_gid != info_b.st_gid) ||
-        (info_a.st_size != info_b.st_size))
+        ((info_a.st_size != info_b.st_size) && !S_ISDIR(info_a.st_mode)))
     {
       return false;
     }
@@ -565,6 +671,100 @@ void SendMsg2Socket(const int fd, const std::string &msg) {
   (void)send(fd, &msg[0], msg.length(), MSG_NOSIGNAL);
 }
 
+/**
+ * Sends the file descriptor passing_fd to the socket socket_fd. Can be used
+ * to transfer an open file descriptor from one process to another. Use
+ * ConnectSocket() to get the socket_fd.
+ */
+bool SendFd2Socket(int socket_fd, int passing_fd) {
+  union {
+    // Make sure that ctrl_msg is properly aligned.
+    struct cmsghdr align;
+    // Buffer large enough to store the file descriptor (ancillary data)
+    unsigned char buf[CMSG_SPACE(sizeof(int))];
+  } ctrl_msg;
+
+  memset(ctrl_msg.buf, 0, sizeof(ctrl_msg.buf));
+
+  struct msghdr msgh;
+  msgh.msg_name = NULL;
+  msgh.msg_namelen = 0;
+
+  unsigned char dummy = 0;
+  struct iovec iov;
+  iov.iov_base = &dummy;
+  iov.iov_len = 1;
+  msgh.msg_iov = &iov;
+  msgh.msg_iovlen = 1;
+
+  msgh.msg_control = ctrl_msg.buf;
+  msgh.msg_controllen = sizeof(ctrl_msg.buf);
+  struct cmsghdr *cmsgp = CMSG_FIRSTHDR(&msgh);
+  cmsgp->cmsg_len = CMSG_LEN(sizeof(int));
+  cmsgp->cmsg_level = SOL_SOCKET;
+  cmsgp->cmsg_type = SCM_RIGHTS;
+  memcpy(CMSG_DATA(cmsgp), &passing_fd, sizeof(int));
+
+  ssize_t retval = sendmsg(socket_fd, &msgh, 0);
+  return (retval != -1);
+}
+
+
+/**
+ * Returns the file descriptor that has been sent with SendFd2Socket. The
+ * msg_fd file descriptor needs to come from a call to accept() on the socket
+ * where the passing file descriptor has been sent to.
+ * Returns -errno on error.
+ */
+int RecvFdFromSocket(int msg_fd) {
+  union {
+    // Make sure that ctrl_msg is properly aligned.
+    struct cmsghdr align;
+    // Buffer large enough to store the file descriptor (ancillary data)
+    unsigned char buf[CMSG_SPACE(sizeof(int))];
+  } ctrl_msg;
+
+  memset(ctrl_msg.buf, 0, sizeof(ctrl_msg.buf));
+
+  struct msghdr msgh;
+  msgh.msg_name = NULL;
+  msgh.msg_namelen = 0;
+
+  unsigned char dummy;
+  struct iovec iov;
+  iov.iov_base = &dummy;
+  iov.iov_len = 1;
+  msgh.msg_iov = &iov;
+  msgh.msg_iovlen = 1;
+
+  msgh.msg_control = ctrl_msg.buf;
+  msgh.msg_controllen = sizeof(ctrl_msg.buf);
+
+  ssize_t retval = recvmsg(msg_fd, &msgh, 0);
+  if (retval == -1)
+    return -errno;
+
+  struct cmsghdr *cmsgp = CMSG_FIRSTHDR(&msgh);
+  assert(cmsgp != NULL);
+  if (cmsgp->cmsg_len != CMSG_LEN(sizeof(int)))
+    return -ERANGE;
+  assert(cmsgp->cmsg_level == SOL_SOCKET);
+  assert(cmsgp->cmsg_type == SCM_RIGHTS);
+
+  int passing_fd;
+  memcpy(&passing_fd, CMSG_DATA(cmsgp), sizeof(int));
+  assert(passing_fd >= 0);
+  return passing_fd;
+}
+
+
+std::string GetHostname() {
+  char name[HOST_NAME_MAX + 1];
+  int retval = gethostname(name, HOST_NAME_MAX);
+  assert(retval == 0);
+  return name;
+}
+
 
 /**
  * set(e){g/u}id wrapper.
@@ -875,7 +1075,7 @@ std::string GetCurrentWorkingDirectory() {
 
 
 /**
- * Helper class that provides callback funtions for the file system traversal.
+ * Helper class that provides callback functions for the file system traversal.
  */
 class RemoveTreeHelper {
  public:
@@ -915,6 +1115,7 @@ bool RemoveTree(const std::string &path) {
   FileSystemTraversal<RemoveTreeHelper> traversal(remove_tree_helper, "",
                                                   true);
   traversal.fn_new_file = &RemoveTreeHelper::RemoveFile;
+  traversal.fn_new_character_dev = &RemoveTreeHelper::RemoveFile;
   traversal.fn_new_symlink = &RemoveTreeHelper::RemoveFile;
   traversal.fn_new_socket = &RemoveTreeHelper::RemoveFile;
   traversal.fn_new_fifo = &RemoveTreeHelper::RemoveFile;
@@ -1012,6 +1213,83 @@ std::vector<std::string> FindDirectories(const std::string &parent_dir) {
 }
 
 
+/**
+ * Finds all files and direct subdirectories under directory (except ., ..).
+ */
+bool ListDirectory(const std::string &directory,
+                   std::vector<std::string> *names,
+                   std::vector<mode_t> *modes)
+{
+  DIR *dirp = opendir(directory.c_str());
+  if (!dirp)
+    return false;
+
+  platform_dirent64 *dirent;
+  while ((dirent = platform_readdir(dirp))) {
+    const std::string name(dirent->d_name);
+    if ((name == ".") || (name == ".."))
+      continue;
+    const std::string path = directory + "/" + name;
+
+    platform_stat64 info;
+    int retval = platform_lstat(path.c_str(), &info);
+    if (retval != 0) {
+      closedir(dirp);
+      return false;
+    }
+
+    names->push_back(name);
+    modes->push_back(info.st_mode);
+  }
+  closedir(dirp);
+
+  SortTeam(names, modes);
+  return true;
+}
+
+
+/**
+ * Looks whether exe is an executable file.  If exe is not an absolute path,
+ * searches the PATH environment.
+ */
+std::string FindExecutable(const std::string &exe) {
+  if (exe.empty())
+    return "";
+
+  std::vector<std::string> search_paths;
+  if (exe[0] == '/') {
+    search_paths.push_back(GetParentPath(exe));
+  } else {
+    char *path_env = getenv("PATH");
+    if (path_env) {
+      search_paths = SplitString(path_env, ':');
+    }
+  }
+
+  for (unsigned i = 0; i < search_paths.size(); ++i) {
+    if (search_paths[i].empty())
+      continue;
+    if (search_paths[i][0] != '/')
+      continue;
+
+    std::string path = search_paths[i] + "/" + GetFileName(exe);
+    platform_stat64 info;
+    int retval = platform_stat(path.c_str(), &info);
+    if (retval != 0)
+      continue;
+    if (!S_ISREG(info.st_mode))
+      continue;
+    retval = access(path.c_str(), X_OK);
+    if (retval != 0)
+      continue;
+
+    return path;
+  }
+
+  return "";
+}
+
+
 std::string GetUserName() {
   struct passwd pwd;
   struct passwd *result = NULL;
@@ -1048,6 +1326,29 @@ std::string GetShell() {
   return shell;
 }
 
+/**
+ * UID -> Name from passwd database
+ */
+bool GetUserNameOf(uid_t uid, std::string *username) {
+  struct passwd pwd;
+  struct passwd *result = NULL;
+  int bufsize = 16 * 1024;
+  char *buf = static_cast<char *>(smalloc(bufsize));
+  while (getpwuid_r(uid, &pwd, buf, bufsize, &result) == ERANGE) {
+    bufsize *= 2;
+    buf = static_cast<char *>(srealloc(buf, bufsize));
+  }
+  if (result == NULL) {
+    free(buf);
+    return false;
+  }
+  if (username)
+    *username = result->pw_name;
+  free(buf);
+  return true;
+}
+
+
 /**
  * Name -> UID from passwd database
  */
@@ -1150,6 +1451,16 @@ std::string GetHomeDirectory() {
   return home_dir;
 }
 
+/**
+ * Returns the output of `uname -m`
+ */
+std::string GetArch() {
+  struct utsname info;
+  int retval = uname(&info);
+  assert(retval == 0);
+  return info.machine;
+}
+
 
 /**
  * Sets soft and hard limit for maximum number of open file descriptors.
@@ -1196,6 +1507,77 @@ void GetLimitNoFile(unsigned *soft_limit, unsigned *hard_limit) {
 }
 
 
+std::vector<LsofEntry> Lsof(const std::string &path) {
+  std::vector<LsofEntry> result;
+
+  std::vector<std::string> proc_names;
+  std::vector<mode_t> proc_modes;
+  ListDirectory("/proc", &proc_names, &proc_modes);
+
+  for (unsigned i = 0; i < proc_names.size(); ++i) {
+    if (!S_ISDIR(proc_modes[i]))
+      continue;
+    if (proc_names[i].find_first_not_of("1234567890") != std::string::npos)
+      continue;
+
+    std::vector<std::string> fd_names;
+    std::vector<mode_t> fd_modes;
+    std::string proc_dir = "/proc/" + proc_names[i];
+    std::string fd_dir   = proc_dir + "/fd";
+    bool rvb = ListDirectory(fd_dir, &fd_names, &fd_modes);
+    uid_t proc_uid = 0;
+
+    // The working directory of the process requires special handling
+    if (rvb) {
+      platform_stat64 info;
+      platform_stat(proc_dir.c_str(), &info);
+      proc_uid = info.st_uid;
+
+      std::string cwd = ReadSymlink(proc_dir + "/cwd");
+      if (HasPrefix(cwd + "/", path + "/", false /* ignore_case */)) {
+        LsofEntry entry;
+        entry.pid = static_cast<pid_t>(String2Uint64(proc_names[i]));
+        entry.owner = proc_uid;
+        entry.read_only = true;  // A bit sloppy but good enough for the moment
+        entry.executable = ReadSymlink(proc_dir + "/exe");
+        entry.path = cwd;
+        result.push_back(entry);
+      }
+    }
+
+    for (unsigned j = 0; j < fd_names.size(); ++j) {
+      if (!S_ISLNK(fd_modes[j]))
+        continue;
+      if (fd_names[j].find_first_not_of("1234567890") != std::string::npos)
+        continue;
+
+      std::string target = ReadSymlink(fd_dir + "/" + fd_names[j]);
+      if (!HasPrefix(target + "/", path + "/", false /* ignore_case */))
+        continue;
+
+      LsofEntry entry;
+      entry.pid = static_cast<pid_t>(String2Uint64(proc_names[i]));
+      entry.owner = proc_uid;
+      entry.read_only = !((fd_modes[j] & S_IWUSR) == S_IWUSR);
+      entry.executable = ReadSymlink(proc_dir + "/exe");
+      entry.path = target;
+      result.push_back(entry);
+    }
+  }
+
+  return result;
+}
+
+
+bool ProcessExists(pid_t pid) {
+  assert(pid > 0);
+  int retval = kill(pid, 0);
+  if (retval == 0)
+    return true;
+  return (errno != ESRCH);
+}
+
+
 /**
  * Blocks a signal for the calling thread.
  */
@@ -1224,9 +1606,11 @@ void WaitForSignal(int signum) {
 
 
 /**
- * Returns -1 of the child crashed or the exit code otherwise
+ * Returns -1 if the child crashed or the exit code otherwise.
+ * @param pid Process identifier.
+ * @param sig_ok List of signals that are still considered a successful termination.
  */
-int WaitForChild(pid_t pid) {
+int WaitForChild(pid_t pid, const std::vector<int> &sig_ok) {
   assert(pid > 0);
   int statloc;
   while (true) {
@@ -1234,16 +1618,83 @@ int WaitForChild(pid_t pid) {
     if (retval == -1) {
       if (errno == EINTR)
         continue;
-      assert(false);
+      PANIC(kLogSyslogErr | kLogDebug,
+            "waitpid failed with errno %d", errno);
     }
     assert(retval == pid);
     break;
   }
   if (WIFEXITED(statloc))
     return WEXITSTATUS(statloc);
+  if (WIFSIGNALED(statloc) && (std::find(sig_ok.begin(), sig_ok.end(),
+                                         WTERMSIG(statloc)) != sig_ok.end()))
+    return 0;
   return -1;
 }
 
+/**
+ * Exec a command as a daemon.
+ */
+
+bool ExecAsDaemon(const std::vector<std::string>  &command_line,
+                       pid_t           *child_pid) {
+  assert(command_line.size() >= 1);
+
+  Pipe<kPipeDetachedChild> pipe_fork;
+  pid_t pid = fork();
+  assert(pid >= 0);
+  if (pid == 0) {
+    pid_t pid_grand_child;
+
+    const char *argv[command_line.size() + 1];
+    for (unsigned i = 0; i < command_line.size(); ++i)
+      argv[i] = command_line[i].c_str();
+    argv[command_line.size()] = NULL;
+    int retval = setsid();
+    assert(retval != -1);
+
+    pid_grand_child = fork();
+    assert(pid_grand_child >= 0);
+
+    if (pid_grand_child != 0){
+      pipe_fork.Write<pid_t>(pid_grand_child);
+      _exit(0);
+    } else {
+      int null_read = open("/dev/null", O_RDONLY);
+      int null_write = open("/dev/null", O_WRONLY);
+      assert((null_read >= 0) && (null_write >= 0));
+      retval = dup2(null_read, 0);
+      assert(retval == 0);
+      retval = dup2(null_write, 1);
+      assert(retval == 1);
+      retval = dup2(null_write, 2);
+      assert(retval == 2);
+      close(null_read);
+      close(null_write);
+
+      execvp(command_line[0].c_str(), const_cast<char **>(argv));
+
+      pipe_fork.CloseWriteFd();
+    }
+  }
+  int statloc;
+  waitpid(pid, &statloc, 0);
+  pid_t buf_child_pid = 0;
+  pipe_fork.Read(&buf_child_pid);
+  if (child_pid != NULL)
+    *child_pid = buf_child_pid;
+  pipe_fork.CloseReadFd();
+
+  LogCvmfs(kLogCvmfs, kLogDebug, "exec'd as daemon %s (PID: %d)",
+           command_line[0].c_str(),
+           static_cast<int>(*child_pid));
+  return true;
+
+}
+
+
+
+
 
 /**
  * Makes a daemon.  The daemon() call is deprecated on OS X
@@ -1310,7 +1761,8 @@ bool ExecuteBinary(
   if (!ManagedExec(cmd_line,
                    preserve_fildes,
                    map_fildes,
-                   true,
+                   true /* drop_credentials */,
+                   false /* clear_env */,
                    double_fork,
                    child_pid))
   {
@@ -1345,7 +1797,7 @@ struct ForkFailures {  // TODO(rmeusel): C++11 (type safe enum)
     kSendPid,
     kUnknown,
     kFailDupFd,
-    kFailGetMaxFd,
+    kFailCloseFds,
     kFailGetFdFlags,
     kFailSetFdFlags,
     kFailDropCredentials,
@@ -1362,8 +1814,8 @@ struct ForkFailures {  // TODO(rmeusel): C++11 (type safe enum)
         return "Unknown Status";
       case kFailDupFd:
         return "Duplicate File Descriptor";
-      case kFailGetMaxFd:
-        return "Read maximal File Descriptor";
+      case kFailCloseFds:
+        return "Close File Descriptors";
       case kFailGetFdFlags:
         return "Read File Descriptor Flags";
       case kFailSetFdFlags:
@@ -1376,6 +1828,82 @@ struct ForkFailures {  // TODO(rmeusel): C++11 (type safe enum)
   }
 };
 
+/**
+ *  Loop through all possible FDs and close them.
+ */
+static bool CloseAllFildesUntilMaxFD(
+  const std::set<int> &preserve_fildes,
+  int max_fd
+) {
+  for (int fd = 0; fd < max_fd; fd++) {
+    if (preserve_fildes.count(fd) == 0) {
+      close(fd);
+    }
+  }
+
+  return true;
+}
+
+/**
+ * Loop through /proc/self/fd and close the listed FDs.
+ * Not used on macOS.
+ */
+#ifndef __APPLE__
+static bool CloseAllFildesInProcSelfFd(const std::set<int> &preserve_fildes)
+{
+  DIR *dirp = opendir("/proc/self/fd");
+  if (!dirp)
+    return false;
+
+  platform_dirent64 *dirent;
+
+  while ((dirent = platform_readdir(dirp))) {
+    const std::string name(dirent->d_name);
+    uint64_t name_uint64;
+
+    // Make sure the dir name is digits only (skips ".", ".." and similar).
+    if (!String2Uint64Parse(name, &name_uint64)) {
+      continue;
+    }
+
+    int fd = static_cast<int>(name_uint64);
+    if (preserve_fildes.count(fd)) {
+      continue;
+    }
+
+    close(fd);
+  }
+
+  closedir(dirp);
+
+  return true;
+}
+#endif
+
+/**
+ * Closes all file descriptors except the ones in preserve_fildes.
+ * To be used after fork but before exec.
+ */
+bool CloseAllFildes(const std::set<int> &preserve_fildes)
+{
+  int max_fd = static_cast<int>(sysconf(_SC_OPEN_MAX));
+  if (max_fd < 0) {
+    return false;
+  }
+
+#ifdef __APPLE__
+  return CloseAllFildesUntilMaxFD(preserve_fildes, max_fd);
+#else   // ifdef __APPLE__
+  if (max_fd > 100000) {
+    // CloseAllFildesUntilMaxFD is inefficient with very large max_fd.
+    // Looping through /proc/self/fd performs better.
+    return CloseAllFildesInProcSelfFd(preserve_fildes);
+  }
+
+  return CloseAllFildesUntilMaxFD(preserve_fildes, max_fd);
+#endif  // #ifdef __APPLE__
+}
+
 /**
  * Execve to the given command line, preserving the given file descriptors.
  * If stdin, stdout, stderr should be preserved, add 0, 1, 2.
@@ -1393,20 +1921,32 @@ bool ManagedExec(const std::vector<std::string>  &command_line,
                  const std::set<int>        &preserve_fildes,
                  const std::map<int, int>   &map_fildes,
                  const bool             drop_credentials,
+                 const bool             clear_env,
                  const bool             double_fork,
                        pid_t           *child_pid)
 {
   assert(command_line.size() >= 1);
 
-  Pipe pipe_fork;
+  Pipe<kPipeDetachedChild> pipe_fork;
   pid_t pid = fork();
   assert(pid >= 0);
   if (pid == 0) {
     pid_t pid_grand_child;
-    int max_fd;
     int fd_flags;
     ForkFailures::Names failed = ForkFailures::kUnknown;
 
+    std::set<int> skip_fds = preserve_fildes;
+    skip_fds.insert(pipe_fork.GetWriteFd());
+
+    if (clear_env) {
+#ifdef __APPLE__
+      environ = NULL;
+#else
+      int retval = clearenv();
+      assert(retval == 0);
+#endif
+    }
+
     const char *argv[command_line.size() + 1];
     for (unsigned i = 0; i < command_line.size(); ++i)
       argv[i] = command_line[i].c_str();
@@ -1424,16 +1964,10 @@ bool ManagedExec(const std::vector<std::string>  &command_line,
     }
 
     // Child, close file descriptors
-    max_fd = sysconf(_SC_OPEN_MAX);
-    if (max_fd < 0) {
-      failed = ForkFailures::kFailGetMaxFd;
+    if (!CloseAllFildes(skip_fds)) {
+      failed = ForkFailures::kFailCloseFds;
       goto fork_failure;
     }
-    for (int fd = 0; fd < max_fd; fd++) {
-      if ((fd != pipe_fork.write_end) && (preserve_fildes.count(fd) == 0)) {
-        close(fd);
-      }
-    }
 
     // Double fork to disconnect from parent
     if (double_fork) {
@@ -1442,13 +1976,13 @@ bool ManagedExec(const std::vector<std::string>  &command_line,
       if (pid_grand_child != 0) _exit(0);
     }
 
-    fd_flags = fcntl(pipe_fork.write_end, F_GETFD);
+    fd_flags = fcntl(pipe_fork.GetWriteFd(), F_GETFD);
     if (fd_flags < 0) {
       failed = ForkFailures::kFailGetFdFlags;
       goto fork_failure;
     }
     fd_flags |= FD_CLOEXEC;
-    if (fcntl(pipe_fork.write_end, F_SETFD, fd_flags) < 0) {
+    if (fcntl(pipe_fork.GetWriteFd(), F_SETFD, fd_flags) < 0) {
       failed = ForkFailures::kFailSetFdFlags;
       goto fork_failure;
     }
@@ -1464,15 +1998,16 @@ bool ManagedExec(const std::vector<std::string>  &command_line,
     // retrieve the PID of the new (grand) child process and send it to the
     // grand father
     pid_grand_child = getpid();
-    pipe_fork.Write(ForkFailures::kSendPid);
-    pipe_fork.Write(pid_grand_child);
+    failed = ForkFailures::kSendPid;
+    pipe_fork.Write<ForkFailures::Names>(failed);
+    pipe_fork.Write<pid_t>(pid_grand_child);
 
     execvp(command_line[0].c_str(), const_cast<char **>(argv));
 
     failed = ForkFailures::kFailExec;
 
    fork_failure:
-    pipe_fork.Write(failed);
+    pipe_fork.Write<ForkFailures::Names>(failed);
     _exit(1);
   }
   if (double_fork) {
@@ -1480,14 +2015,14 @@ bool ManagedExec(const std::vector<std::string>  &command_line,
     waitpid(pid, &statloc, 0);
   }
 
-  close(pipe_fork.write_end);
+  pipe_fork.CloseWriteFd();
 
   // Either the PID or a return value is sent
   ForkFailures::Names status_code;
-  bool retcode = pipe_fork.Read(&status_code);
-  assert(retcode);
+  pipe_fork.Read<ForkFailures::Names>(&status_code);
+
   if (status_code != ForkFailures::kSendPid) {
-    close(pipe_fork.read_end);
+    pipe_fork.CloseReadFd();
     LogCvmfs(kLogCvmfs, kLogDebug, "managed execve failed (%s)",
              ForkFailures::ToString(status_code).c_str());
     return false;
@@ -1496,11 +2031,10 @@ bool ManagedExec(const std::vector<std::string>  &command_line,
   // read the PID of the spawned process if requested
   // (the actual read needs to be done in any case!)
   pid_t buf_child_pid = 0;
-  retcode = pipe_fork.Read(&buf_child_pid);
-  assert(retcode);
+  pipe_fork.Read(&buf_child_pid);
   if (child_pid != NULL)
     *child_pid = buf_child_pid;
-  close(pipe_fork.read_end);
+  pipe_fork.CloseReadFd();
   LogCvmfs(kLogCvmfs, kLogDebug, "execve'd %s (PID: %d)",
            command_line[0].c_str(),
            static_cast<int>(buf_child_pid));
@@ -1548,7 +2082,8 @@ bool SafeWriteV(int fd, struct iovec *iov, unsigned iovcnt) {
   unsigned iov_idx = 0;
 
   while (nbytes) {
-    ssize_t retval = writev(fd, &iov[iov_idx], iovcnt - iov_idx);
+    ssize_t retval =
+      writev(fd, &iov[iov_idx], static_cast<int>(iovcnt - iov_idx));
     if (retval < 0) {
       if (errno == EINTR)
         continue;
