@@ -3,7 +3,6 @@ package cmd
 import (
 	"encoding/json"
 	"fmt"
-	"os"
 
 	log "github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
@@ -22,15 +21,14 @@ func init() {
 }
 
 var downloadManifestCmd = &cobra.Command{
-	Use:     "download-manifest",
-	Short:   "Download the manifest of the image, if sucessful it will print the manifest itself, otherwise will show what went wrong.",
+	Use:     "download-manifest <image>",
+	Short:   "Download the manifest of the image, if successful it will print the manifest itself, otherwise will show what went wrong.",
 	Aliases: []string{"get-manifest"},
 	Args:    cobra.MinimumNArgs(1),
-	Run: func(cmd *cobra.Command, args []string) {
+	RunE: func(cmd *cobra.Command, args []string) (err error) {
 		img, err := lib.ParseImage(args[0])
 		if err != nil {
-			fmt.Println(err)
-			os.Exit(1)
+			return err
 		}
 		if img.Tag == "" && img.Digest == "" {
 			log.Fatal("Please provide either the image tag or the image digest")
@@ -42,11 +40,14 @@ var downloadManifestCmd = &cobra.Command{
 		manifest, err := img.GetManifest()
 		if err != nil {
 			l.LogE(err).Fatal("Error in getting the manifest")
+			return err
 		}
 		text, err := json.MarshalIndent(manifest, "", "  ")
 		if err != nil {
 			l.LogE(err).Fatal("Error in encoding the manifest as JSON")
+			return err
 		}
 		fmt.Println(string(text))
+		return nil
 	},
 }
