@@ -117,7 +117,7 @@ func CreateSymlinkIntoCVMFS(CVMFSRepo, newLinkName, toLinkPath string) (err erro
 		return err
 	}
 	// from the relativePath we remove the first part of the path.
-	// The part we remove reprensent the same directory where is the target.
+	// The part we remove represents the same directory where is the target.
 	linkChunks := strings.Split(relativePath, string(os.PathSeparator))
 	link := filepath.Join(linkChunks[1:]...)
 
@@ -137,7 +137,7 @@ func CreateSymlinkIntoCVMFS(CVMFSRepo, newLinkName, toLinkPath string) (err erro
 				// the file exists and it is a symlink, we overwrite it
 				err = os.Remove(newLinkName)
 				if err != nil {
-					err = fmt.Errorf("Error in removing existsing symlink: %s", err)
+					err = fmt.Errorf("Error in removing existing symlink: %s", err)
 					llog(l.LogE(err)).Error("Error in removing previous symlink")
 					return err
 				}
@@ -362,7 +362,7 @@ func RemoveSingularityImageFromManifest(CVMFSRepo string, manifest da.Manifest)
 	}
 	err := RemoveDirectory(CVMFSRepo, manifest.GetSingularityPath())
 	if err != nil {
-		llog(l.LogE(err)).Error("Error in removing singularity direcotry")
+		llog(l.LogE(err)).Error("Error in removing singularity directory")
 		return err
 	}
 	return nil
@@ -405,7 +405,7 @@ func LayerMetadataPath(CVMFSRepo, layerDigest string) string {
 	return filepath.Join(LayerPath(CVMFSRepo, layerDigest), ".metadata")
 }
 
-//from /cvmfs/$REPO/foo/bar -> foo/bar
+// from /cvmfs/$REPO/foo/bar -> foo/bar
 func TrimCVMFSRepoPrefix(path string) string {
 	return strings.Join(strings.Split(path, string(os.PathSeparator))[3:], string(os.PathSeparator))
 }
@@ -482,7 +482,7 @@ func CreateCatalogIntoDir(CVMFSRepo, dir string) (err error) {
 	return nil
 }
 
-//writes data to file and publish in cvmfs repo path
+// writes data to file and publish in cvmfs repo path
 func WriteDataToCvmfs(CVMFSRepo, path string, data []byte) (err error) {
 	tmpFile, err := temp.UserDefinedTempFile()
 	if err != nil {
@@ -556,7 +556,7 @@ func CreateSneakyChain(CVMFSRepo, newChainId, previousChainId string, layer tar.
 			}
 
 			if len(sourceDirs) != len(destinationDirs) {
-				return fmt.Errorf("Different number of directories between the source and tha target directories during a template transaction. source: %s , # of dir: %d, target: %s, # of dirs: %d", source, len(sourceDirs), destination, len(destinationDirs))
+				return fmt.Errorf("Different number of directories between the source and the target directories during a template transaction. source: %s , # of dir: %d, target: %s, # of dirs: %d", source, len(sourceDirs), destination, len(destinationDirs))
 			}
 
 			f, _ := os.OpenFile(filepath.Join(destination, ".cvmfscatalog"), os.O_CREATE|os.O_RDONLY, constants.FilePermision)
@@ -697,9 +697,12 @@ func CreateSneakyChain(CVMFSRepo, newChainId, previousChainId string, layer tar.
 				l.LogE(err).Error("Error in chmod")
 				return err
 			}
-			if err := os.Chown(path, header.Uid, header.Gid); err != nil {
-				l.LogE(err).Error("Error in chown")
-				return err
+			// TODO(vavolkl): do we need to chown?
+			if os.Getenv("CVMFS_DUCC_NO_CHOWN") == "" {
+				if err := os.Chown(path, header.Uid, header.Gid); err != nil {
+					l.LogE(err).Error("Error in chown")
+					return err
+				}
 			}
 			if err := os.Chtimes(path, header.AccessTime, header.ModTime); err != nil {
 				l.LogE(err).Error("Error in chtimes")
