@@ -16,6 +16,7 @@ import (
 	"sync"
 	"time"
 
+	"github.com/aoliveti/curling"
 	"github.com/docker/docker/image"
 	"github.com/olekukonko/tablewriter"
 	log "github.com/sirupsen/logrus"
@@ -77,7 +78,7 @@ func SetupRegistries() {
 
 		if ident == "" || ((user == "" || pass == "") && proxy == "") {
 			log.Fatalf("missing either $%s, ($%s or $%s) or %s for %s",
-			           iEnv, uEnv, uPass, proxyEnv, r)
+				iEnv, uEnv, uPass, proxyEnv, r)
 		}
 
 		inputRegistries = append(inputRegistries, RegistryConfig{
@@ -277,7 +278,7 @@ func (img *Image) GetOCIImage() (config image.Image, err error) {
 	client := &http.Client{}
 	req, err := http.NewRequest("GET", configUrl, nil)
 	if err != nil {
-		l.LogE(err).Warning("Impossible to create a request for getting the changes no chnages set.")
+		l.LogE(err).Warning("Impossible to create a request for getting the changes, no changes set.")
 		return
 	}
 	req.Header.Set("Authorization", token)
@@ -467,14 +468,12 @@ func (i *Image) GetPublicSymlinkPath() string {
 
 func (img *Image) getByteManifestList() ([]byte, error) {
 	url := img.GetManifestUrl("")
-	return makeGetRequest(url, map[string]string{"Accept":
-		"application/vnd.docker.distribution.manifest.list.v2+json, application/vnd.oci.image.index.v1+json"})
+	return makeGetRequest(url, map[string]string{"Accept": "application/vnd.docker.distribution.manifest.list.v2+json, application/vnd.oci.image.index.v1+json"})
 }
 
 func (img *Image) getByteManifest(reference string) ([]byte, error) {
 	url := img.GetManifestUrl(reference)
-	return makeGetRequest(url, map[string]string{"Accept":
-		"application/vnd.docker.distribution.manifest.v2+json, application/vnd.oci.image.manifest.v1+json"})
+	return makeGetRequest(url, map[string]string{"Accept": "application/vnd.docker.distribution.manifest.v2+json, application/vnd.oci.image.manifest.v1+json"})
 }
 
 func GetAuthToken(url string, credentials []Credentials) (token string, err error) {
@@ -505,7 +504,7 @@ func firstRequestForAuth_internal(url, user, pass string) (token string, err err
 	}
 	if resp.StatusCode != 401 {
 		log.WithFields(log.Fields{
-			"url": url,
+			"url":         url,
 			"status code": resp.StatusCode,
 		}).Info("Expected status code 401.")
 		return "", err
@@ -631,7 +630,7 @@ func (img *Image) GetLayers(layersChan chan<- downloadedLayer, manifestChan chan
 	// at this point we iterate each layer and we download it.
 	for _, layer := range manifest.Layers {
 		if layer.MediaType == "application/vnd.docker.image.rootfs.foreign.diff.tar.gzip" {
-			continue;
+			continue
 		}
 
 		wg.Add(1)
@@ -831,7 +830,7 @@ func (ld *LayerDownloader) getToken() (token string, err error) {
 	firstLayer := manifest.Layers[0]
 	for _, l := range manifest.Layers {
 		if l.MediaType == "application/vnd.docker.image.rootfs.foreign.diff.tar.gzip" {
-			continue;
+			continue
 		}
 		firstLayer = l
 		break
@@ -900,7 +899,7 @@ func (img *Image) CreateSneakyChainStructure(CVMFSRepo string) (err error, lastC
 
 	paths := []string{}
 	for _, chain := range chainIDs {
-		if (chain == "") {
+		if chain == "" {
 			continue
 		}
 		path := cvmfs.ChainPath(CVMFSRepo, chain.String())
@@ -958,7 +957,7 @@ func (img *Image) CreateSneakyChainStructure(CVMFSRepo string) (err error, lastC
 	ld := NewLayerDownloader(img)
 	previous := ""
 	for i, chain := range chainIDs {
-		if (chain == "") {
+		if chain == "" {
 			continue
 		}
 		digest := chain.String()
@@ -1078,6 +1077,15 @@ func makeGetRequest(url string, headers map[string]string) ([]byte, error) {
 		req.Header.Set(k, v)
 	}
 
+	// for debugging: log curl command corresponding to request
+	if log.IsLevelEnabled(log.TraceLevel) {
+		curlcmd, err := curling.NewFromRequest(req)
+		if err != nil {
+			log.Fatal(err)
+		}
+		log.Trace(curlcmd)
+	}
+
 	resp, err := client.Do(req)
 	if err != nil {
 		l.LogE(err).Error("Error in making the HTTP request")
