@@ -7,12 +7,14 @@
 
 #include <errno.h>
 #include <sys/select.h>
+#include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/uio.h>
 #include <sys/wait.h>
 #ifdef __APPLE__
 #include <sys/sysctl.h>
 #endif
+#include <sys/un.h>
 #include <unistd.h>
 
 #include <cstdio>
@@ -20,10 +22,10 @@
 #include <string>
 #include <vector>
 
-#include "logging.h"
 #include "options.h"
-#include "platform.h"
 #include "sanitizer.h"
+#include "util/logging.h"
+#include "util/platform.h"
 #include "util/posix.h"
 #include "util/string.h"
 
@@ -66,26 +68,38 @@ static string MkFqrn(const string &repository) {
   return repository;
 }
 
+#if defined(__APPLE__) && !defined(USE_MACFUSE_KEXT)
+static bool IsFuseTInstalled() {
+  return true;
+  string fuseTComponentsPaths[] = { "/usr/local/bin/go-nfsv4", 
+                                          "/usr/local/lib/libfuse-t.dylib",
+                                           "/usr/local/lib/libfuse-t.a" };
+  const int pathsNumber = sizeof(fuseTComponentsPaths) / sizeof(fuseTComponentsPaths[0]);
+  platform_stat64 info;
+  for (int idx = 0; idx < pathsNumber; ++idx) {
+    bzero(&info, sizeof(platform_stat64));
+    int retval = platform_stat(fuseTComponentsPaths[idx].c_str(), &info);
+    if ((retval != 0) || !S_ISREG(info.st_mode)) {
+      return false;
+    }
+  }
+  return true;
+}
+#endif
 
 static bool CheckFuse() {
+#if defined(__APPLE__) && !defined(USE_MACFUSE_KEXT)
+  bool is_fuse_t_installed = IsFuseTInstalled();
+  if (!is_fuse_t_installed) {
+    LogCvmfs(kLogCvmfs, kLogStderr, "FUSE-T installation check failed. FUSE not loaded"); 
+  }
+  return is_fuse_t_installed;
+#else 
   string fuse_device;
   int retval;
 #ifdef __APPLE__
-  if (FileExists("/Library/Filesystems/osxfuse.fs/Contents/Resources/"
-                 "load_osxfuse"))
-  {
-    // OS X Fuse 3
-    retval = system("/Library/Filesystems/osxfuse.fs/Contents/Resources/"
-                    "load_osxfuse");
-  } else {
-    retval = system("/Library/Filesystems/osxfusefs.fs/Support/load_osxfusefs");
-  }
-  if (retval != 0) {
-    LogCvmfs(kLogCvmfs, kLogStderr, "Failed loading OSX Fuse");
-    return false;
-  }
-  fuse_device = "/dev/osxfuse0";
-#else
+  fuse_device = "/dev/macfuse0";
+#else 
   fuse_device = "/dev/fuse";
 #endif
   platform_stat64 info;
@@ -95,6 +109,7 @@ static bool CheckFuse() {
     return false;
   }
   return true;
+#endif
 }
 
 
@@ -131,7 +146,7 @@ static bool CheckStrictMount(const string &fqrn) {
 static bool CheckProxy() {
   string param;
   int retval = options_manager_.GetValue("CVMFS_HTTP_PROXY", &param);
-  if (!retval) {
+  if (!retval || param.empty()) {
     LogCvmfs(kLogCvmfs, kLogStderr, "CVMFS_HTTP_PROXY required");
     return false;
   }
@@ -142,6 +157,7 @@ static bool CheckProxy() {
 static bool CheckConcurrentMount(const string &fqrn,
                                  const string &workspace,
                                  string *mountpointp) {
+  LockFile(workspace + "/cvmfs_io." + fqrn + ".mountcheck.lock");
   // Try connecting to cvmfs_io socket
   int socket_fd = ConnectSocket(workspace + "/cvmfs_io." + fqrn);
   if (socket_fd < 0)
@@ -160,6 +176,60 @@ static bool CheckConcurrentMount(const string &fqrn,
   return true;
 }
 
+static int GetExistingFuseFd(
+  const string &fqrn, const string &workspace, uid_t cvmfs_uid)
+{
+  // Try connecting to cvmfs_io socket
+  int talk_fd = ConnectSocket(workspace + "/cvmfs_io." + fqrn);
+  if (talk_fd < 0)
+    return -1;
+
+  // Create temporary socket
+  std::string recv_sock_dir = CreateTempDir(workspace + "/fusefd");
+  if (recv_sock_dir.empty() || (chmod(recv_sock_dir.c_str(), 0755) != 0)) {
+    close(talk_fd);
+    return -1;
+  }
+  std::string recv_sock_path = recv_sock_dir + "/sock";
+  int recv_sock_fd = MakeSocket(recv_sock_path, 0660);
+  if ((recv_sock_fd < 0) ||
+      (chown(recv_sock_path.c_str(), cvmfs_uid, getegid()) != 0))
+  {
+    if (recv_sock_fd >= 0)
+      close(recv_sock_fd);
+    close(talk_fd);
+    unlink(recv_sock_path.c_str());
+    rmdir(recv_sock_dir.c_str());
+    return -1;
+  }
+  listen(recv_sock_fd, 1);
+
+  // Trigger fd transfer
+  SendMsg2Socket(talk_fd, "send mount fd " + recv_sock_path);
+  string result;
+  char buf;
+  while (read(talk_fd, &buf, 1) == 1) {
+    if (buf != '\n')
+      result.push_back(buf);
+  }
+  close(talk_fd);
+
+  int fuse_fd = -1;
+  if (result == "OK") {
+    struct sockaddr_un addr;
+    unsigned int len = sizeof(addr);
+    int con_fd =
+      accept(recv_sock_fd, reinterpret_cast<struct sockaddr *>(&addr), &len);
+    fuse_fd = RecvFdFromSocket(con_fd);
+    close(con_fd);
+  }
+  close(recv_sock_fd);
+  unlink(recv_sock_path.c_str());
+  rmdir(recv_sock_dir.c_str());
+
+  return fuse_fd;
+}
+
 
 static bool GetCacheDir(const string &fqrn, string *cachedir) {
   string param;
@@ -269,6 +339,39 @@ static std::string GetCvmfsBinary() {
   return result;
 }
 
+static int AttachMount(const std::string &mountpoint, const std::string &fqrn,
+                       int fuse_fd)
+{
+#ifdef __APPLE__
+  (void) mountpoint;
+  (void) fqrn;
+  (void) fuse_fd;
+  return 1;
+#else
+  platform_stat64 info;
+  int retval = platform_stat(mountpoint.c_str(), &info);
+  if (retval != 0)
+    return 1;
+
+  char mntopt[100];
+  snprintf(mntopt, sizeof(mntopt),
+           "ro,fd=%i,rootmode=%o,user_id=%d,group_id=%d",
+           fuse_fd, info.st_mode & S_IFMT, geteuid(), getegid());
+  // TODO(jblomer): remove NOSUID according to options
+  retval = mount("cvmfs2", mountpoint.c_str(), "fuse",
+                 MS_NODEV | MS_RDONLY | MS_NOSUID, mntopt);
+  if (retval != 0) {
+    LogCvmfs(kLogCvmfs, kLogStderr,
+             "Cannot attach to existing fuse module (%d)", errno);
+    return 1;
+  }
+  LogCvmfs(kLogCvmfs, kLogStdout | kLogSyslog,
+           "CernVM-FS: linking %s to repository %s (attaching)",
+           mountpoint.c_str(), fqrn.c_str());
+  return 0;
+#endif
+}
+
 
 int main(int argc, char **argv) {
   bool dry_run = false;
@@ -328,6 +431,17 @@ int main(int argc, char **argv) {
     new DefaultOptionsTemplateManager(fqrn));
   options_manager_.ParseDefault(fqrn);
 
+  string optarg;
+  if (options_manager_.GetValue("CVMFS_SYSLOG_LEVEL", &optarg))
+    SetLogSyslogLevel(String2Uint64(optarg));
+  if (options_manager_.GetValue("CVMFS_SYSLOG_FACILITY", &optarg))
+    SetLogSyslogFacility(String2Int64(optarg));
+  if (options_manager_.GetValue("CVMFS_USYSLOG", &optarg))
+    SetLogMicroSyslog(optarg + ".mount");
+  if (options_manager_.GetValue("CVMFS_DEBUGLOG", &optarg))
+    SetLogDebugFile(optarg + ".mount");
+  SetLogSyslogPrefix(fqrn);
+
   int retval;
   int sysret;
   bool dedicated_cachedir = false;
@@ -349,29 +463,6 @@ int main(int argc, char **argv) {
   if (!retval) return 1;
   retval = CheckProxy();
   if (!retval) return 1;
-  // This is not a sure thing.  When the CVMFS_CACHE_BASE parameter is changed
-  // two repositories can get mounted concurrently (but that should not hurt).
-  // If the same repository is mounted multiple times at the same time, there
-  // is a race here.  Eventually, only one repository will be mounted while the
-  // other cvmfs processes block on a file lock in the cache.
-  string prev_mountpoint;
-  retval = CheckConcurrentMount(fqrn, workspace, &prev_mountpoint);
-  if (retval) {
-    if (remount && (mountpoint == prev_mountpoint)) {
-      // Actually remounting is too hard, but pretend that it worked
-      return 0;
-    }
-    LogCvmfs(kLogCvmfs, kLogStderr, "Repository %s is already mounted on %s",
-             fqrn.c_str(), prev_mountpoint.c_str());
-    return 1;
-  } else {
-    // No double mount
-    if (remount) {
-      LogCvmfs(kLogCvmfs, kLogStderr, "Repository %s is not mounted on %s",
-               fqrn.c_str(), mountpoint.c_str());
-      return 1;
-    }
-  }
 
   // Retrieve cvmfs uid/gid and fuse gid if exists
   uid_t uid_cvmfs;
@@ -386,13 +477,52 @@ int main(int argc, char **argv) {
   }
   has_fuse_group = GetGidOf("fuse", &gid_fuse);
 
-  // Prepare workspace and cache directory
+  // Prepare workspace
   retval = MkdirDeep(workspace, 0755, false);
   if (!retval) {
     LogCvmfs(kLogCvmfs, kLogStderr, "Failed to create workspace %s",
              workspace.c_str());
     return 1;
   }
+
+  // This is not a sure thing.  When the CVMFS_CACHE_BASE parameter is changed
+  // two repositories can get mounted concurrently (but that should not hurt).
+  // If the same repository is mounted multiple times at the same time, there
+  // is a race here.  Eventually, only one repository will be mounted while the
+  // other cvmfs processes block on a file lock in the cache.
+  string prev_mountpoint;
+  retval = CheckConcurrentMount(fqrn, workspace, &prev_mountpoint);
+  if (retval) {
+    if (remount && (mountpoint == prev_mountpoint)) {
+      // Actually remounting is too hard, but pretend that it worked
+      return 0;
+    }
+    // Identify zombie fuse processes that are held open by other mount
+    // namespaces
+    if ((mountpoint == prev_mountpoint) && !IsMountPoint(mountpoint)) {
+      // Allow for group access to the socket receiving the fuse fd
+      umask(007);
+      int fuse_fd = GetExistingFuseFd(fqrn, workspace, uid_cvmfs);
+      if (fuse_fd < 0) {
+        LogCvmfs(kLogCvmfs, kLogStderr,
+                 "Cannot connect to existing fuse module");
+        return 1;
+      }
+      return AttachMount(mountpoint, fqrn, fuse_fd);
+    }
+    LogCvmfs(kLogCvmfs, kLogStderr, "Repository %s is already mounted on %s",
+             fqrn.c_str(), prev_mountpoint.c_str());
+    return 1;
+  } else {
+    // No double mount
+    if (remount) {
+      LogCvmfs(kLogCvmfs, kLogStderr, "Repository %s is not mounted on %s",
+               fqrn.c_str(), mountpoint.c_str());
+      return 1;
+    }
+  }
+
+  // Prepare cache directory
   if (dedicated_cachedir) {
     retval = MkdirDeep(cachedir, 0755, false);
     if (!retval) {
@@ -537,21 +667,25 @@ int main(int argc, char **argv) {
   fd_set readfds;
   bool stdout_open = true;
   bool stderr_open = true;
+  int status = 0;
+  int ended = false;
+
   do {
     FD_ZERO(&readfds);
     if (stdout_open) FD_SET(fd_stdout, &readfds);
     if (stderr_open) FD_SET(fd_stderr, &readfds);
-    do {
-      retval = select(nfds, &readfds, NULL, NULL, NULL);
-      if ((retval == -1) && (errno != EINTR)) {
-        LogCvmfs(kLogCvmfs, kLogStderr, "Failed to pipe stdout/stderr");
-        return 32;
-      }
-      if (retval > 0)
-        break;
-    } while (true);
+
+    struct timeval timeout;
+    timeout.tv_sec = 0;
+    timeout.tv_usec = 100;
+    retval = select(nfds, &readfds, NULL, NULL, &timeout);
+    if ((retval == -1) && (errno != EINTR)) {
+      LogCvmfs(kLogCvmfs, kLogStderr, "Failed to pipe stdout/stderr");
+      return 32;
+    }
+
     char buf;
-    int num_bytes;
+    ssize_t num_bytes;
     if (FD_ISSET(fd_stdout, &readfds)) {
       num_bytes = read(fd_stdout, &buf, 1);
       switch (num_bytes) {
@@ -580,16 +714,21 @@ int main(int argc, char **argv) {
           return 32;
       }
     }
-  } while (stdout_open || stderr_open);
+
+    // TODO(vvolkl): This block has seen several rounds of iterations,
+    // that tried to adress races and incorrect return codes.
+    // Should be looked at once again to make sure the cause of the
+    // race is understood, and if the timeout is really needed.
+
+    ended = (waitpid(pid_cvmfs, &status, WNOHANG) == pid_cvmfs);
+  } while ((stdout_open || stderr_open) && !ended);
+  if (!ended) {
+    waitpid(pid_cvmfs, &status, 0);
+  }
+
   close(fd_stdout);
   close(fd_stderr);
 
-  int status;
-  retval = waitpid(pid_cvmfs, &status, 0);
-  if (retval == -1) {
-    LogCvmfs(kLogCvmfs, kLogStderr, "Failed reading return code");
-    return 32;
-  }
   if (WIFEXITED(status) && (WEXITSTATUS(status) == 0))
     return 0;
 
