@@ -1,21 +1,29 @@
-#!/usr/bin/python
-
-import BaseHTTPServer
-import SimpleHTTPServer
-import SocketServer
+#!/usr/bin/env python
+try:
+  import http.server as BaseHTTPServer
+  import socketserver as SocketServer
+except ImportError:
+  import BaseHTTPServer
+  import SimpleHTTPServer
+  import SocketServer
 import sys
 import os
 from optparse import OptionParser
 
+
+
 class FaultyHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
     def do_GET(self):
         self.send_response(400)
+        self.end_headers()
 
     def do_POST(self):
         self.send_response(400)
+        self.end_headers()
 
     def do_HEAD(self):
         self.send_response(400)
+        self.end_headers()
 
 
 parser = OptionParser()
@@ -28,7 +36,7 @@ if not options.http_port:
     parser.print_help()
     sys.exit(1)
 
-print "start serving..."
+print("start serving...")
 handler = FaultyHTTPRequestHandler
 httpd = SocketServer.TCPServer(("", options.http_port), handler)
 httpd.serve_forever()
