@@ -2,8 +2,8 @@
  * This file is part of the CernVM File System.
  */
 
-#ifndef TEST_UNITTESTS_TESTUTIL_H_
-#define TEST_UNITTESTS_TESTUTIL_H_
+#ifndef TEST_COMMON_TESTUTIL_H_
+#define TEST_COMMON_TESTUTIL_H_
 
 #include <gtest/gtest.h>
 
@@ -16,16 +16,17 @@
 #include <string>
 #include <vector>
 
-#include "atomic.h"
 #include "catalog_mgr.h"
+#include "crypto/hash.h"
 #include "directory_entry.h"
-#include "hash.h"
-#include "ingestion/ingestion_source.h"
 #include "history.h"
+#include "ingestion/ingestion_source.h"
 #include "object_fetcher.h"
 #include "upload_facility.h"
+#include "util/atomic.h"
 
 pid_t GetParentPid(const pid_t pid);
+std::string GetProcessname(const pid_t pid);
 std::string GetExecutablePath(const std::string &exe_name);
 
 unsigned GetNoUsedFds();
@@ -179,6 +180,10 @@ class AbstractMockUploader : public upload::AbstractUploader {
     assert(AbstractMockUploader::not_implemented);
   }
 
+  virtual bool Mkdir(const std::string &path) {
+    assert(AbstractMockUploader::not_implemented);
+  }
+
   virtual bool PlaceBootstrappingShortcut(const shash::Any &object) {
     assert(AbstractMockUploader::not_implemented);
   }
@@ -311,7 +316,7 @@ class MockCatalog : public MockObjectStorage<MockCatalog> {
  public:
   static const std::string rhs;
   static const shash::Any  root_hash;
-  static unsigned int      instances;
+  static atomic_int32      instances;
 
  public:
   struct NestedCatalog {
@@ -395,7 +400,7 @@ class MockCatalog : public MockObjectStorage<MockCatalog> {
     string name = root_path.substr(pos + 1, string::npos);
     File mountpoint_file(shash::Any(), 4096, parent_path, name);
     files_.push_back(mountpoint_file);
-    ++MockCatalog::instances;
+    atomic_inc32(&MockCatalog::instances);
   }
 
   MockCatalog(const MockCatalog &other) :
@@ -407,11 +412,11 @@ class MockCatalog : public MockObjectStorage<MockCatalog> {
     children_(other.children_), files_(other.files_),
     chunks_(other.chunks_)
   {
-    ++MockCatalog::instances;
+    atomic_inc32(&MockCatalog::instances);
   }
 
   ~MockCatalog() {
-    --MockCatalog::instances;
+    atomic_dec32(&MockCatalog::instances);
   }
 
   /**
@@ -454,7 +459,8 @@ class MockCatalog : public MockObjectStorage<MockCatalog> {
   bool LookupPath(const PathString &path,
                   catalog::DirectoryEntry *dirent) const;
   bool ListingPath(const PathString &path,
-                   catalog::DirectoryEntryList *listing) const;
+                   catalog::DirectoryEntryList *listing,
+                   const bool expand_symlink) const;
 
   bool GetVOMSAuthz(std::string *authz) { return false; }
 
@@ -563,10 +569,8 @@ class MockCatalogManager : public AbstractCatalogManager<MockCatalog> {
 
   virtual ~MockCatalogManager() { delete spooler_; }
 
-  virtual LoadError LoadCatalog(const PathString &mountpoint,
-                                const shash::Any &hash,
-                                std::string  *catalog_path,
-                                shash::Any   *catalog_hash);
+  virtual LoadReturn GetNewRootCatalogContext(CatalogContext *result);
+  virtual LoadReturn LoadCatalogByHash(CatalogContext *ctlg_context);
 
   virtual MockCatalog* CreateCatalog(const PathString  &mountpoint,
                                  const shash::Any  &catalog_hash,
@@ -684,7 +688,6 @@ class MockHistory : public history::History,
   bool GetByName(const std::string &name, Tag *tag) const;
   bool GetByDate(const time_t timestamp, Tag *tag) const;
   bool List(std::vector<Tag> *tags) const;
-  bool Tips(std::vector<Tag> *channel_tips) const;
 
   bool GetBranchHead(const std::string &branch_name, Tag *tag) const;
   bool InsertBranch(const Branch &branch);
@@ -721,16 +724,6 @@ class MockHistory : public history::History,
     return tag.root_hash;
   }
 
-  static bool gt_channel_revision(const Tag &lhs, const Tag &rhs) {
-    return (lhs.channel == rhs.channel)
-              ? lhs.revision > rhs.revision
-              : lhs.channel > rhs.channel;
-  }
-
-  static bool eq_channel(const Tag &lhs, const Tag &rhs) {
-    return lhs.channel == rhs.channel;
-  }
-
   static bool gt_hashes(const Tag &lhs, const Tag &rhs) {
     return lhs.root_hash > rhs.root_hash;
   }
@@ -753,8 +746,7 @@ class MockHistory : public history::History,
       , inverse_(inverse)
     { }
     bool operator()(const Tag &tag) const {
-      const bool p = (tag.revision > tag_.revision || tag.name == tag_.name) &&
-                      tag.channel == tag_.channel;
+      const bool p = (tag.revision > tag_.revision || tag.name == tag_.name);
       return inverse_ ^ p;
     }
     const Tag  &tag_;
@@ -868,4 +860,4 @@ class MockObjectFetcher : public AbstractObjectFetcher<MockObjectFetcher> {
 };
 
 
-#endif  // TEST_UNITTESTS_TESTUTIL_H_
+#endif  // TEST_COMMON_TESTUTIL_H_
