@@ -24,7 +24,7 @@
  * └── [   0]  fuz
  */
 
-const std::string four_empty_files =
+static const char *four_empty_files =
 "YmFyAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
 "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAADAwMDY2NCAAMDAxNzUw"
 "IAAwMDE3NTAgADAwMDAwMDAwMDAwIDEzMjQxNTIwNTIxIDAxMTUxMAAgMAAAAAAAAAAAAAAAAAAA"
@@ -88,7 +88,7 @@ const std::string four_empty_files =
  *
  */
 
-const std::string simple_tar =
+static const char *simple_tar =
 "dGFyL2FhYS8AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
 "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAADAwMDc3NSAAMDAxNzUw"
 "IAAwMDE3NTAgADAwMDAwMDAwMDAwIDEzMjM2NjExNDczIDAxMjM1NgAgNQAAAAAAAAAAAAAAAAAA"
@@ -181,7 +181,7 @@ const std::string simple_tar =
  *
  */
 
-const std::string complex_tar =
+static const char *complex_tar =
 "YmFyAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
 "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAADAwMDY2NCAAMDAxNzUw"
 "IAAwMDE3NTAgADAwMDAwMDAwMDAwIDEzMjQxNTU2MDAxIDAxMTUxMgAgMAAAAAAAAAAAAAAAAAAA"
