@@ -9,8 +9,8 @@
 
 #include "catalog.h"
 #include "catalog_rw.h"
-#include "compression.h"
-#include "hash.h"
+#include "compression/compression.h"
+#include "crypto/hash.h"
 #include "shortstring.h"
 #include "testutil.h"
 #include "util/posix.h"
