@@ -6,8 +6,8 @@
 
 #include <fcntl.h>
 
-#include "compression.h"
-#include "hash.h"
+#include "compression/compression.h"
+#include "crypto/hash.h"
 
 TEST(T_Compression, CompressFd2Null) {
   shash::Any hash(shash::kSha1);
