@@ -15,9 +15,9 @@
 #include <map>
 #include <string>
 
-#include "fs_traversal.h"
-#include "platform.h"
 #include "util/file_guard.h"
+#include "util/fs_traversal.h"
+#include "util/platform.h"
 #include "util/posix.h"
 
 class T_FsTraversal : public ::testing::Test {
