@@ -17,17 +17,17 @@
 #include "catalog_mgr_client.h"
 #include "catalog_mgr_rw.h"
 #include "catalog_test_tools.h"
+#include "crypto/signature.h"
 #include "history_sqlite.h"
 #include "manifest.h"
 #include "mountpoint.h"
 #include "options.h"
-#include "signature.h"
 #include "testutil.h"
 #include "upload.h"
 #include "upload_spooler_definition.h"
 #include "util/pointer.h"
 #include "util/posix.h"
-#include "uuid.h"
+#include "util/uuid.h"
 
 using namespace std;  // NOLINT
 
@@ -170,6 +170,7 @@ TEST_F(T_MountPoint, TriageCacheMgr) {
 TEST_F(T_MountPoint, RamCacheMgr) {
   options_mgr_.SetValue("CVMFS_CACHE_PRIMARY", "ram");
   options_mgr_.SetValue("CVMFS_CACHE_ram_TYPE", "ram");
+  options_mgr_.SetValue("CVMFS_CACHE_ram_SIZE", "75");
   {
     UniquePtr<FileSystem> fs(FileSystem::Create(fs_info_));
     EXPECT_EQ(loader::kFailOk, fs->boot_status());
@@ -247,7 +248,9 @@ TEST_F(T_MountPoint, TieredComplex) {
   options_mgr_.SetValue("CVMFS_CACHE_tiered_upper_UPPER", "uu_ram");
   options_mgr_.SetValue("CVMFS_CACHE_tiered_upper_LOWER", "ul_ram");
   options_mgr_.SetValue("CVMFS_CACHE_uu_ram_TYPE", "ram");
+  options_mgr_.SetValue("CVMFS_CACHE_uu_ram_SIZE", "75");
   options_mgr_.SetValue("CVMFS_CACHE_ul_ram_TYPE", "ram");
+  options_mgr_.SetValue("CVMFS_CACHE_ul_ram_SIZE", "75");
   options_mgr_.SetValue("CVMFS_CACHE_tiered_LOWER", "tiered_lower");
   options_mgr_.SetValue("CVMFS_CACHE_tiered_lower_TYPE", "tiered");
   options_mgr_.SetValue("CVMFS_CACHE_tiered_lower_UPPER", "lu_posix");
@@ -408,7 +411,7 @@ TEST_F(T_MountPoint, LockWorkspace) {
     case -1:
       abort();
     case 0:
-      read(pipe_sync[0], &pid, sizeof(pid));
+      SafeRead(pipe_sync[0], &pid, sizeof(pid));
       UniquePtr<FileSystem> fs01(FileSystem::Create(fs_info_));
       switch (fs01->boot_status()) {
         case loader::kFailOk:
@@ -421,7 +424,7 @@ TEST_F(T_MountPoint, LockWorkspace) {
   }
 
   UniquePtr<FileSystem> fs01(FileSystem::Create(fs_info_));
-  write(pipe_sync[1], &pid, sizeof(pid));
+  SafeWrite(pipe_sync[1], &pid, sizeof(pid));
   int stat_loc;
   int retval = waitpid(pid, &stat_loc, 0);
   EXPECT_NE(retval, -1);
@@ -448,15 +451,6 @@ TEST_F(T_MountPoint, UuidCache) {
 }
 
 
-TEST_F(T_MountPoint, QuotaMgr) {
-  // Fails because the unit test binary cannot become a quota manager process
-  options_mgr_.SetValue("CVMFS_SHARED_CACHE", "yes");
-  {
-    UniquePtr<FileSystem> fs(FileSystem::Create(fs_info_));
-    EXPECT_EQ(loader::kFailQuota, fs->boot_status());
-  }
-}
-
 TEST_F(T_MountPoint, MountLatest) {
   CreateMiniRepository(&options_mgr_, &repo_path_);
   ASSERT_TRUE(HasSuffix(repo_path_, "repo", false));
